# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from subprocess import getstatusoutput
    pythoncmd="python3"
except:
    from commands import getstatusoutput
    pythoncmd="python2"

from django.core.management.base import BaseCommand, CommandError

from codenerix.lib.debugger import Debugger

class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Clean memcache"
    
    def handle(self, *args, **options):
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        
        # Get environment
        status, output = getstatusoutput("echo 'flush_all' | nc localhost 11211")
        if status: raise CommandError(output)




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import operator
import base64
import hashlib
from functools import reduce

from django.db.models import Q
from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from codenerix.middleware import get_current_user
from django.conf import settings

from codenerix.helpers import clean_memcache_item
from codenerix.models import GenLog, CodenerixModel


class GenPerson(GenLog, models.Model):  # META: Abstract class
    '''
    Defines a person
    '''
    # Control fields
    user = models.OneToOneField(User, blank=True, null=True, related_name='person')
    name = models.CharField(_("Name"), max_length=45, blank=False, null=False)
    surname = models.CharField(_("Surname"), max_length=90, blank=False, null=False)
    disabled = models.DateTimeField(_("Disabled from"), null=True, blank=True)
    creator = models.ForeignKey(User, blank=True, null=True, related_name='creators', default=None)

    class Meta(CodenerixModel.Meta):
        abstract = True

    def __unicode__(self):
        if self.name and self.surname:
            output = '%s %s' % (smart_text(self.name), smart_text(self.surname))
        elif self.name:
            output = self.name
        elif self.surname:
            output = self.surname
        else:
            output = "%s*" % (self.user)
        return smart_text(output)

    def __str__(self):
        return self.__unicode__()

    def __fields__(self, info):
        fields = []
        fields.append(('user__username', _('User')))
        fields.append(('name', _('Name'), 100))
        fields.append(('surname', _('Surname'), 100))
        fields.append(('user__email', _('Email'), 100))
        fields.append(('user__date_joined', _('Created'), 80))
        fields.append(('user__is_active', _('Active'), None, 'center'))
        fields.append(('disabled', _('Disabled from')))
        return fields

    def __limitQ__(self, info):
        limit = {}
        # If user is not a superuser, the shown records depends on the profile
        if not info.request.user.is_superuser:

            # All the people records shown below must be related with this user
            criterials = []

            # The criterials are not exclusives
            if criterials:
                limit["profile_people_limit"] = reduce(operator.or_, criterials)

        return limit

    def __searchF__(self, info):
        tf = {}
        tf['user__is_active'] = (_('Active'), lambda x: Q(user__is_active=x), [(True, _('Yes')), (False, _('No'))])
        return tf

    def __searchQ__(self, info, text):
        tf = {}
        tf['username'] = Q(user__username__icontains=text.lower())
        tf['name'] = Q(name__icontains=text)
        tf['surname'] = Q(surname__icontains=text)
        tf['user_email'] = Q(user__email__icontains=text)
        tf['user__date_joined'] = "datetime"
        return tf

    def is_admin(self):
        try:
            return bool(self.user.is_superuser or self.user.groups.get(name='Admins'))
        except Exception:
            return False

    def profiles(self):
        '''
        return the rolls this people is related with
        '''
        limit = []

        if self.is_admin():
            limit.append(_("Administrator"))
        limit.sort()

        return limit

    def delete(self):
        self.clean_memcache()
        return super(GenPerson, self).delete()

    def lock_delete(self, request=None):
        if request is not None:
            if request.user == self.user:
                return _("Cannot delete to yourself")
            else:
                return super(GenPerson, self).lock_delete()
        else:
            return super(GenPerson, self).lock_delete()

    def clean_memcache(self):
        if self.pk:
            prefix = hashlib.md5()
            prefix.update(base64.b64encode(settings.SECRET_KEY.encode('utf-8')))
            clean_memcache_item("person:{}".format(self.pk), prefix.hexdigest())

    def get_grouppermit(self):
        return self.user.groups.all()

    def presave(self, username, password, email, confirm):
        self.clean_memcache()
        # Check username
        # Rewrite username
        username = username.lower()

        # if exist self.user no modify username
        if not self.user and not username:
            raise ValidationError("This person doesn't have a user assigned, can not change password or email without an user")

        # Check passwords
        if password and confirm and password != confirm:
            raise ValidationError("Passwords do not match")
        if password and (len(password) < settings.PASSWORD_MIN_SIZE):
            raise ValidationError("Password can not be smaller than {0} characters".format(settings.PASSWORD_MIN_SIZE))

        # Check email
        if email:
            validate_email(email)
        else:
            raise ValidationError("Missing email when saving a new person without a preassigned user")

        # Create user and save it to the database
        if self.user:
            # Update password if any
            if password:
                self.user.set_password(password)
            # Update email address
            self.user.email = email
        else:
            # Create new user
            self.user = User.objects.create_user(username=username, email=email, password=password)
            # user creator of the user
            self.creator = get_current_user()

        # Save changes
        self.user.save()

    def refresh_permissions(self):

        # Check we have a user to work with
        if self.user:

            # Clear groups and permisions for this user
            self.user.groups.clear()
            self.user.user_permissions.clear()

            # Collect all groups and unique permissions for this user relationships
            groups = []
            permissions = []
            for x in self._meta.get_fields():
                model = x.related_model

                # Only check roles
                if model and issubclass(model, GenRole):
                    # Get the link
                    link = getattr(self, x.name, None)

                    # Check if the linked class has CodenerixMeta
                    if link and hasattr(link, 'CodenerixMeta'):

                        # Get groups and permissions from that class
                        groups += list(getattr(link.CodenerixMeta, 'rol_groups', None) or {})
                        permissions += list(getattr(link.CodenerixMeta, 'rol_permissions', None) or [])

            # Add groups
            for groupname in set(groups):
                group = Group.objects.filter(name=groupname).first()
                if group is None:
                    # Group not found, remake permissions for all groups with roles
                    GenPerson.group_permissions(type(self))
                    # Check again
                    group = Group.objects.filter(name=groupname).first()
                    if group is None:
                        raise IOError("Group '{} not found in the system. I have tried to remake groups but this group is not defined as a Role and it doesn't belong to the system either".format(groupname))

                # Add the user to this group
                self.user.groups.add(group)

            # Add permissions
            for permissionname in set(permissions):
                permission = Permission.objects.filter(codename=permissionname).first()
                if permission is None:
                    raise IOError("Permission '{}' not found in the system".format(permissionname))

                # Add the permission to this user
                self.user.user_permissions.add(permission)

        else:
            raise IOError("You can not refresh permissions for a Person wich doesn't have an associated user")

    @staticmethod
    def group_permissions(clss):

        groupsresult = {}
        for x in clss._meta.get_fields():
            model = x.related_model

            # Check if it is a role
            if model and issubclass(model, GenRole):

                # Check if the class has CodenerixMeta
                if hasattr(model, 'CodenerixMeta'):

                    # Get groups and permissions
                    groups = getattr(model.CodenerixMeta, 'rol_groups', [])

                    # Add groups
                    if groups:
                        groups_is_dict = type(groups) is dict
                        groupslist = list(set(groups))
                        for groupname in groupslist:

                            # Check permissions just in case something is wrong
                            perms = []
                            if groups_is_dict:
                                for permname in groups[groupname]:
                                    perm = Permission.objects.filter(codename=permname).first()
                                    # Commented by Juan Soler on a project we
                                    # both are working on. Since this code is
                                    # preventing a proper execution, I will
                                    # comment it as he has done in our project
                                    # and I will wait for him to validate this lines
                                    # if perm is None:
                                    #    raise IOError("Permission '{}' not found for group '{}'!".format(permname, groupname))
                                    # else:
                                    if perm is not None:
                                        perms.append(perm)

                            # Remember perms for this group
                            if groupname not in groupsresult:
                                groupsresult[groupname] = []
                            groupsresult[groupname] += perms

        # Set permissions for all groups
        for groupname in groupsresult:
            # Get group
            group = Group.objects.filter(name=groupname).first()
            if group is None:
                # Add group if doesn't exists
                group = Group(name=groupname)
                group.save()
            else:
                # Remove all permissions for this group
                group.permissions.clear()

            # Add permissions to the group
            for perm in groupsresult[groupname]:
                group.permissions.add(perm)


class GenRole(object):
    def save(self, *args, **kwards):
        # only update permissions for new users
        REFRESH_PERMISSIONS = not self.pk
        result = super(GenRole, self).save(*args, **kwards)
        if REFRESH_PERMISSIONS:
            person = self.__CDNX_search_person_CDNX__()
            if person:
                person.refresh_permissions()
        return result

    def delete(self):
        person = self.__CDNX_search_person_CDNX__()
        result = super(GenRole, self).delete()
        if person:
            # update permissions
            person.refresh_permissions()
        return result

    def __CDNX_search_person_CDNX__(self):
        # search relation with GenPerson
        person = None
        for field in self._meta.get_fields():
            model = field.related_model
            if model and issubclass(model, GenPerson):
                try:
                    person = getattr(self, field.name)
                except ObjectDoesNotExist:
                    pass
                break
        return person

    def CDNX_refresh_permissions_CDNX(self):
        person = self.__CDNX_search_person_CDNX__()
        if person:
            person.refresh_permissions()



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe

from codenerix import __version__


def codenerix(request):
    '''
    Codenerix CONTEXT
    '''
    # Get values
    DEBUG = getattr(settings, 'DEBUG', False)
    VERSION = getattr(settings, 'VERSION', _('WARNING: No version set to this code, add VERSION contant to your configuration'))
    
    # Set environment
    return {
        'DEBUG': DEBUG,
        'VERSION': VERSION,
        'CODENERIX_VERSION': __version__,
    }


def codenerix_js( request ):
    cnf = {}
    
    # Get values
    CONNECTION_ERROR = getattr(settings,'CONNECTION_ERROR', None)
    ALARMS_LOOPTIME  = getattr(settings,'ALARMS_LOOPTIME',  None)
    ALARMS_QUICKLOOP = getattr(settings,'ALARMS_QUICKLOOP', None)
    ALARMS_ERRORLOOP = getattr(settings,'ALARMS_ERRORLOOP', None)
    DEBUG = getattr(settings,'DEBUG', False)
    DATERANGEPICKER_OPTIONS = getattr(settings, 'DATERANGEPICKER_OPTIONS', None)
    DATETIME_RANGE_FORMAT = getattr(settings, 'DATETIME_RANGE_FORMAT', None)
    
    # Set values
    if CONNECTION_ERROR is not None: cnf['connection_error'] = CONNECTION_ERROR
    if ALARMS_LOOPTIME  is not None: cnf['alarms_looptime']  = ALARMS_LOOPTIME
    if ALARMS_QUICKLOOP is not None: cnf['alarms_quickloop'] = ALARMS_QUICKLOOP
    if ALARMS_ERRORLOOP is not None: cnf['alarms_errorloop'] = ALARMS_ERRORLOOP
    cnf['debug'] = str(DEBUG).lower()
    cnf['session_cookie_age'] = getattr(settings,'SESSION_COOKIE_AGE', None)
    cnf['codenerix_css'] = getattr(settings,'CODENERIX_CSS', _('WARNING: CODENERIX_CSS is not set in your configuration!!!'))
    cnf['codenerix_js']  = getattr(settings,'CODENERIX_JS', _('WARNING: CODENERIX_JS is not set in your configuration!!!'))
    
    # Set daterange
    if (DATERANGEPICKER_OPTIONS is None) or (DATETIME_RANGE_FORMAT is None):
        daterangepicker = '"'
        if DATERANGEPICKER_OPTIONS is None:
            daterangepicker+= "{}".format(_(' WARNING: DATERANGEPICKER_OPTIONS is not set in your configuration!!! '))
        if DATETIME_RANGE_FORMAT is None:
            daterangepicker+= "{}".format(_(' WARNING: DATETIME_RANGE_FORMAT is not set in your configuration!!! '))
        daterangepicker+= '"'
        cnf['daterangepicker'] = mark_safe(daterangepicker)
    else:
        cnf['daterangepicker'] = mark_safe(settings.DATERANGEPICKER_OPTIONS.format(
                    Format=settings.DATETIME_RANGE_FORMAT[1],
                    From= _("From"),
                    To= _("To"),
                    Apply= _("Apply"),
                    Cancel= _("Cancel"),
                    Su= _("Su"),
                    Mo= _("Mo"),
                    Tu= _("Tu"),
                    We= _("We"),
                    Th= _("Th"),
                    Fr= _("Fr"),
                    Sa= _("Sa"),
                    January= _("January"),
                    February= _("February"),
                    March= _('March'),
                    April= _('April'),
                    May= _('May'),
                    June= _('June'),
                    July= _('July'), 
                    August= _('August'), 
                    September= _('September'), 
                    October= _('October'),
                    November= _('November'), 
                    December= _('December')
                    ))
    
    # Return environment
    return {'cnf': cnf }




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

try:
    from subprocess import getstatusoutput
    pythoncmd="python3"
except:
    from commands import getstatusoutput
    pythoncmd="python2"

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from codenerix.lib.debugger import Debugger

class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Do a touch to the project"
    
    def add_arguments(self, parser):
        
        # Named (optional) arguments
        parser.add_argument('-f',
            action='store_true',
            dest='f',
            default=False,
            help='Keep the command working forever')
        
        # Named (optional) arguments
        parser.add_argument('--follow',
            action='store_true',
            dest='follow',
            default=False,
            help='Keep the command working forever')
    
    def handle(self, *args, **options):
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        
        # Get environment
        appname=settings.ROOT_URLCONF.split(".")[0]
        basedir=settings.BASE_DIR
        appdir = os.path.abspath("{}/{}".format(basedir,appname))
        
        # While keep working
        keepworking=True
        while keepworking:
            
            # Collect
            self.debug("Collecting...",color='blue')
            status, output = getstatusoutput("{}/manage collectstatic --noinput".format(appdir))
            if status: raise CommandError(output)
            for line in output.split("\n"):
                if line[0:7]=="Copying":
                    path=line.split("'")[1].replace(appdir,".")
                    self.debug("    > {}".format(path))
                elif ("static file copied to" in line) or ("static files copied to" in line):
                    done=line.split(" ")[0]
                    linesp=line.split(",")
                    if len(linesp)==1:
                        total=linesp[0].split(" ")[0]
                    else:
                        total=linesp[1].split(" ")[1]
                    self.debug("{}/{} files copied".format(done,total),color='cyan')
                elif "was already registered. Reloading models is not advised as it can lead to inconsistencies, most notably with related models." in line:
                    pass
                elif "new_class._meta.apps.register_model(new_class._meta.app_label, new_class)" in line:
                    pass
                elif line=="":
                    pass
                else:
                    self.warning("Unknown string: #{}#".format(line))
            
            # Clean
            self.debug("Cleaning...",color='blue')
            status, output = getstatusoutput("{}/manage clean".format(appdir))
            if status: raise CommandError(output)
            
            # Touch
            self.debug("Touch...",color='blue', tail=False)
            filenames = os.listdir(appdir)
            filenames.sort()
            for name in filenames:
                if name[0:4]=='wsgi' or name[-4:]=='wsgi':
                    status, output = getstatusoutput("/usr/bin/touch {}/{}".format(appdir, name))
                    if status: raise CommandError(output)
                    self.debug(" [{}]".format(name),color='cyan', header=False, tail=False)
            self.debug(" Done",color='green', header=False)
            
            if options['follow'] or options['f']:
                # Ask the user if would really like to remove all locale folders
                self.debug("Hit a ENTER when ready to go or Q to exit (ENTER|q) ",tail=False, color="purple")
                try:
                    key = raw_input().lower()
                except NameError:
                    key = input().lower()
                except KeyboardInterrupt:
                    self.debug(" ",header=False)
                    key='q'
                self.debug("", header=False)
                self.debug(" ")
                if key=='q':
                    self.debug("Quitting!",color='yellow')
                    keepworking=False
            else:
                self.debug("Quitting!",color='yellow')
                keepworking=False




# from django.conf import settings
from haystack.backends.elasticsearch2_backend import Elasticsearch2SearchBackend, Elasticsearch2SearchEngine

class AsciifoldingElasticBackend(Elasticsearch2SearchBackend):
    '''
    Mounir Messelmeni - 13/December/2015
    https://mounirmesselmeni.github.io/2015/12/13/enable-asciifolding-in-elasticsearchhaystack/
    '''

    def __init__(self, *args, **kwargs):
        super(AsciifoldingElasticBackend, self).__init__(*args, **kwargs)
        analyzer = {
            "ascii_analyser" : {
                "tokenizer" : "standard",
                "filter" : ["standard", "asciifolding", "lowercase"]
            },
            "ngram_analyzer": {
                "type": "custom",
                "tokenizer": "lowercase",
                "filter": ["haystack_ngram", "asciifolding"]
            },
            "edgengram_analyzer": {
                "type": "custom",
                "tokenizer": "lowercase",
                "filter": ["haystack_edgengram", "asciifolding"]
            }
        }
        self.DEFAULT_SETTINGS['settings']['analysis']['analyzer'] = analyzer

    def build_schema(self, fields):
        content_field_name, mapping = super(AsciifoldingElasticBackend,
                                            self).build_schema(fields)

        for field_name, field_class in fields.items():
            field_mapping = mapping[field_class.index_fieldname]

            if field_mapping['type'] == 'string' and field_class.indexed:
                if not hasattr(field_class, 'facet_for') and not field_class.field_type in('ngram', 'edge_ngram'):
                    field_mapping['analyzer'] = "ascii_analyser"

            mapping.update({field_class.index_fieldname: field_mapping})
        return (content_field_name, mapping)


class AsciifoldingElasticSearchEngine(Elasticsearch2SearchEngine):
    '''
    Mounir Messelmeni - 13/December/2015
    https://mounirmesselmeni.github.io/2015/12/13/enable-asciifolding-in-elasticsearchhaystack/
    '''
    backend = AsciifoldingElasticBackend



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Debugger helps to debug the system
'''

__version__ = "2017082500"

__all__ = ['Debugger', 'lineno']

import time
import datetime
import inspect

from codenerix.lib.colors import colors


def lineno():
    '''
    Returns the current line number in our program.
    '''
    return inspect.currentframe().f_back.f_lineno


class Debugger(object):

    __indebug = {}
    __inname = None

    def __autoconfig(self):
        # Define debug configuration
        import sys
        debugger = {}
        debugger['screen'] = (sys.stdout, ['*'])
        # debugger['log'] = (open("log/debug.log","a"), ['*'] )
        self.set_debug(debugger)

    def set_debug(self, debug=None):
        if debug is None:
            self.__autoconfig()
        else:
            if type(debug) is dict:
                # Set the deepness system
                idebug = debug.copy()
                if 'deepness' in debug:
                    if debug['deepness']:
                        idebug['deepness'] -= 1
                    else:
                        idebug = {}

                # Save internal debugger
                self.__indebug = idebug
            else:
                raise IOError("Argument is not a dictionary")

    def get_debug(self):
        return self.__indebug

    def set_name(self, name):
        self.__inname = name

    def get_name(self):
        return self.__inname

    def color(self, color):
        # Colors$
        if color in colors:
            (darkbit, subcolor) = colors[color]
            return u"\033[%1d;%02dm" % (darkbit, subcolor)
        else:
            if color:
                self.debug(u"\033[1;31mColor '%s' unknown\033[1;00m\n" % (color))
            return ''

    def debug(self, msg=None, header=None, color=None, tail=None, head=None, footer=None):
        # Allow better names for debug calls
        if header is None:
            if head is None:
                header = True
            else:
                header = head
        if tail is None:
            if footer is None:
                tail = True
            else:
                tail = footer

        # Retrieve the name of the class
        clname = self.__class__.__name__

        # Retrieve tabular
        if 'tabular' in self.__indebug:
            tabular = self.__indebug['tabular']
        else:
            tabular = ''

        # For each element inside indebug
        for name in self.__indebug:

            # If this is no deepeness key, keep going
            if name not in ['deepness', 'tabular']:

                # Get color
                if name != 'screen':
                    color = None
                color_ini = self.color(color)
                color_end = self.color('close')

                # Get file output handler and indebug list
                (handler, indebug) = self.__indebug[name]
                if msg and type(handler) == str:
                    # Open handler buffer
                    handlerbuf = open(handler, "a")
                else:
                    handlerbuf = handler

                # Look up if the name of the class is inside indebug
                if (clname in indebug) or (('*' in indebug) and ('-%s' % (clname) not in indebug)):

                    # Set line head name
                    if self.__inname:
                        headname = self.__inname
                    else:
                        headname = clname

                    # Build the message
                    message = color_ini
                    if header:
                        now = datetime.datetime.fromtimestamp(time.time())
                        message += "%02d/%02d/%d %02d:%02d:%02d %-15s - %s" % (now.day, now.month, now.year, now.hour, now.minute, now.second, headname, tabular)
                    if msg:
                        try:
                            message += str(msg)
                        except UnicodeEncodeError:
                            message += str(msg.encode('ascii', 'ignore'))
                    message += color_end
                    if tail:
                        message += '\n'

                    # Print it on the buffer handler
                    if msg:
                        handlerbuf.write(message)
                        handlerbuf.flush()
                    else:
                        # If we shouldn't show the output, say to the caller we should output something
                        return True

                # Autoclose handler when done
                if msg and type(handler) == str:
                    handlerbuf.close()

        # If we shouldn't show the output
        if not msg:
            # Say to the caller we shouldn't output anything
            return False

    def warning(self, msg, header=True, tail=True):
        self.warningerror(msg, header, 'WARNING', 'yellow', tail)

    def error(self, msg, header=True, tail=True):
        self.warningerror(msg, header, 'ERROR', 'red', tail)

    def warningerror(self, msg, header, prefix, color, tail):

        # Retrieve the name of the class
        clname = self.__class__.__name__

        # Retrieve tabular
        if 'tabular' in self.__indebug:
            tabular = self.__indebug['tabular']
        else:
            tabular = ''

        # For each element inside indebug
        for name in self.__indebug:

            # If this is no deepeness key, keep going
            if name not in ['deepness', 'tabular']:

                # Get file output handler and indebug list
                (handler, indebug) = self.__indebug[name]
                if type(handler) == str:
                    # Open handler buffer
                    handlerbuf = open(handler, "a")
                else:
                    handlerbuf = handler

                # Get color
                if name != 'screen':
                    color = None
                color_ini = self.color(color)
                color_end = self.color('close')

                # Build the message
                message = color_ini
                if header:
                    # Set line head name
                    if self.__inname:
                        headname = self.__inname
                    else:
                        headname = clname

                    now = datetime.datetime.fromtimestamp(time.time())
                    message += "\n%s - %02d/%02d/%d %02d:%02d:%02d %-15s - %s" % (prefix, now.day, now.month, now.year, now.hour, now.minute, now.second, headname, tabular)
                if msg:
                    try:
                        message += str(msg)
                    except UnicodeEncodeError:
                        message += str(msg.encode('ascii', 'ignore'))
                message += color_end
                if tail:
                    message += '\n'

                # Print it on the buffer handler
                handlerbuf.write(message)
                handlerbuf.flush()

                # Autoclose handler when done
                if type(handler) == str:
                    handlerbuf.close()



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from threading import local

from django.shortcuts import redirect
from django.conf import settings


class SecureRequiredMiddleware(object):
    def __init__(self, get_response=None):
        self.get_response = get_response
        self.paths = getattr(settings, 'HTTPS_PATHS', getattr(settings, 'SECURE_REQUIRED_PATHS', ('/', )))
        self.enabled = self.paths and getattr(settings, 'HTTPS_SUPPORT', True)
    
    def process_request(self, request):
        
        if self.enabled and not request.is_secure():

            # Check for negative matching, URLs we don't want to be secure
            for path in self.paths:
                full_path = request.get_full_path()
                if path[0] == '-' and (full_path.startswith(path[1:]) or re.compile(path[1:]).match(full_path)):
                    return None
                
            # Check for positive matching, URLs we want to be secure
            for path in self.paths:
                full_path = request.get_full_path()
                if full_path.startswith(path) or re.compile(path).match(full_path):
                    request_url = request.build_absolute_uri(request.get_full_path())
                    secure_url = request_url.replace('http://', 'https://')
                    return redirect(secure_url, permanent=True)

        # Any other lateral case
        return None
    
    def __call__(self, request):
        # Code to be executed for each request before the view (and later middleware) are called.
        response = self.process_request(request)
        
        # Get response
        if response is None:
            response = self.get_response(request)
        
        # Code to be executed for each request/response after the view is called
        # ... pass ...
        
        # Return response
        return response


class CurrentUserMiddleware(object):
    '''
    Let the system to have the username everywhere
    '''
    def __init__(self, get_response=None):
        self.get_response = get_response
    
    def process_request(self, request):
        _user.value = request.user
    
    def __call__(self, request):
        # Code to be executed for each request before the view (and later middleware) are called.
        self.process_request(request)
        
        # Get response
        response = self.get_response(request)
        
        # Code to be executed for each request/response after the view is called
        # ... pass ...
        
        # Return response
        return response
    
def get_current_user():
    '''
    Thought this function you can get the user loged in everywhere
    '''
    if 'value' in dir(_user):
        return _user.value
    else:
        return None
_user = local()




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.core.management.base import BaseCommand

from codenerix.lib.debugger import Debugger

try:
    from subprocess import getstatusoutput
    pythoncmd="python3"
except:
    from commands import getstatusoutput
    pythoncmd="python2"

class Command(BaseCommand, Debugger):
    
    # IDENTS
    packages = {
            'KERNEL':       'django-codenerix',
            'PAYMENTS':     'django-codenerix-payments',
            'TRANSPORTS':   'django-codenerix-transports',
            'PQPROCLIENT':  'django-codenerix-pqpro-client',
            'PQPROSERVER':  'django-codenerix-pqpro-server',
            'EXTENSIONS':   'django-codenerix-extensions',
            'INVOICING':    'django-codenerix-invoicing',
            'PRODUCTS':     'django-codenerix-products',
            'SLIDER':       'django-codenerix-slider',
            'STORAGES':     'django-codenerix-storages',
        }
    
    # Dependencies
    imports = [
            #("bcrypt",                          "import bcrypt"),
            ("bson",                            "from bson import json_util"),
            ("django-angular==0.8.4",           "from djng.forms.angular_base import TupleErrorList"),
            #("Crypto.Cipher",                   "from Crypto.Cipher import AES"),
            #("cryptography",                    "import cryptography"),
            ("dateutil",                        "from dateutil.tz import tzutc"),
            ("django-multi-email-field==0.5",   "from multi_email_field.forms import MultiEmailField", "pip install git+https://github.com/fle/django-multi-email-field.git"),
            ("django-recaptcha",                "from captcha import client"),
            ("django-rosetta",                  "import rosetta"),
            ("jsonfield",                       "import jsonfield"),
            ("ldap3",                           "import ldap3"),
            ("openpyxl==2.2.5",                 "import openpyxl"),
            #("paypalrestsdk",                   "import paypalrestsdk"),
            ("Pillow",                          "import PIL"),
            #("pycrypto==2.6.1",                 "from Crypto.Cipher import AES"),
            ("python-dateutil",                 "import dateutil.parser"),
            #("scipy",                           "import scipy"),
            ("Unidecode",                       "from unidecode import unidecode"),
            ("xhtml2pdf",                       "import xhtml2pdf", "git clone https://github.com/xhtml2pdf/xhtml2pdf - and, python2 setup.py install or python3 setup.py install - sudo apt-get remove python-setuptools - wget https://bootstrap.pypa.io/get-pip.py - sudo -H python get-pip.py - sudo -H pip install -U pip setuptools"),
            ]
    
    # Show this when the user types help
    help = "Do a touch to the project"
    
    def handle(self, *args, **options):
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        self.check_dependencies()
    
    def check_dependencies(self):
        self.debug("Checking all libraries required for CODENERIX to work:",color='cyan')
        
        # Django Angular
        for entry in self.imports:
            if len(entry)==2:
                (name,command)=entry
                helptext=None
            else:
                (name,command,helptext)=entry
            self.debug("    > {:32s} :: ".format(name),color='blue', tail=None),
            error, output = getstatusoutput("{} -c '{}' 2>&1".format(pythoncmd,command))
            if not error:
                # Library test passed, check version
                package = name.split("=")[0]
                error, output = getstatusoutput("pip freeze | grep -e '^{}=' || echo 'EMPTY'".format(package))
                if not error:
                    if output=='EMPTY':
                        self.debug("OK",color='green', header=None, tail=None)
                        self.debug(" (BUILT-IN)",color='purple', header=None)
                    elif "error" in output.lower():
                        self.debug("ERROR - OUTPUT WAS:\n".format(name),color='red', header=None, tail=None)
                        if output.split("\n")[-1]=='EMPTY':
                            output = "\n".join(output.split("\n")[0:-1])
                        self.debug(output,color='white', header=None)
                    else:
                        if output<name:
                            self.debug("OLDER".format(name),color='yellow', header=None, tail=None)
                            if helptext:
                                self.debug(" - HELP FOR YOU: {}".format(helptext),color='white', header=None, tail=None)
                            self.debug(" ",header=None)
                        else:
                            self.debug("OK".format(name),color='green', header=None)
                else:
                    self.debug("ERROR while looking for the package in PIP for '{}'".format(name),color='red')
                    self.debug(" -> Test: pip freeze | grep -e '^{}=' || echo 'EMPTY'".format(package),color='yellow')
                    self.debug("pip freeze | grep '{}' || echo 'EMPTY'".format(name))
                    self.error(error)
                    self.debug("OUTPUT: {}".format(output))
                    if helptext:
                        self.debug("HELP FOR YOU: {}".format(helptext))
                    print
            else:
                self.debug("MISSING or wrong version for '{}'".format(name),color='red')
                self.debug(" -> Test: {}".format(command),color='yellow')
                self.debug("{} -c '{}'".format(pythoncmd,command))
                self.error(error)
                self.debug("OUTPUT: {}".format(output))
                self.debug("")




# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
from PIL import Image, ImageDraw, ImageFont
from os import path

from django.template import Library
from django.conf import settings

from django import template
from django.template import Variable, NodeList
from django.contrib.auth.models import Group

register = Library()

@register.filter
def txt2img(text, FontSize=14, bg="#ffffff", fg="#000000", font="FreeMono.ttf"):
    font_dir = settings.MEDIA_ROOT + "/txt2img/"   # Set the directory to store the images
    img_name_temp = text + "-" + bg.strip("#") + "-" + fg.strip("#") + "-" + str(FontSize)   # Remove hashes
    try:
        # python 2.7
        img_name_encode = hashlib.md5(img_name_temp).hexdigest()
    except TypeError:
        # python 3.x
        img_name_temp = bytes(img_name_temp, encoding='utf-8')
        img_name_encode = hashlib.md5(img_name_temp).hexdigest()

    img_name = "%s.jpg" % (img_name_encode)
    
    if path.exists(font_dir + img_name):   # Make sure img doesn't exist already
        pass
    else:
        font_size = FontSize
        fnt = ImageFont.truetype(font_dir + font, font_size)
        w, h = fnt.getsize(text)
        img = Image.new('RGBA', (w, h), bg)
        draw = ImageDraw.Draw(img)
        draw.fontmode = "0"
        draw.text((0, 0), text, font=fnt, fill=fg)
        img.save(font_dir + img_name, "JPEG", quality=100)
    imgtag = '<img src="' + settings.MEDIA_URL + 'txt2img/' + img_name + '" alt="' + text + '" />'
    return imgtag

@register.tag
def ifusergroup(parser, token):
    """ Check to see if the currently logged in user belongs to a specific
    group. Requires the Django authentication contrib app and middleware.

    Usage: {% ifusergroup Admins %} ... {% endifusergroup %}, or
           {% ifusergroup Admins Clients Sellers %} ... {% else %} ... {% endifusergroup %}

    """
    try:
        tokensp = token.split_contents()
        groups = []
        groups+=tokensp[1:]
    except ValueError:
        raise template.TemplateSyntaxError("Tag 'ifusergroup' requires at least 1 argument.")
    
    nodelist_true = parser.parse(('else', 'endifusergroup'))
    token = parser.next_token()
    
    if token.contents == 'else':
        nodelist_false = parser.parse(tuple(['endifusergroup',]))
        parser.delete_first_token()
    else:
        nodelist_false = NodeList()
    
    return GroupCheckNode(groups, nodelist_true, nodelist_false)


class GroupCheckNode(template.Node):
    def __init__(self, groups, nodelist_true, nodelist_false):
        self.groups = groups
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
    def render(self, context):
        user = Variable('user').resolve(context)
        
        if not user.is_authenticated():
            return self.nodelist_false.render(context)
        
        allowed=False
        for checkgroup in self.groups:
            try:
                group = Group.objects.get(name=checkgroup)
            except Group.DoesNotExist:
                break
                
            if group in user.groups.all():
                allowed=True
                break
        
        if allowed:
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import random
import hashlib
import io
import base64
from datetime import datetime
import json
import imghdr

from django import forms
from django.core.urlresolvers import reverse, resolve
from django.utils.translation import ugettext as _
from django.core.files.base import File
from django.utils import formats
from django.utils.encoding import smart_text
from django.utils.html import escapejs
from django.conf import settings
from django.utils.translation import get_language

from captcha.widgets import ReCaptcha

from codenerix.helpers import get_class


class StaticSelectMulti(forms.widgets.Select):
    __language = None

    def __init__(self, *args, **kwargs):
        targs = None
        if args:
            targs = args[0]
        if kwargs:
            targs = kwargs
        if targs:
            # common
            self.is_required = targs.get('is_required', False)
            self.form_name = targs.get('form_name', '')
            self.field_name = targs.get('field_name', '')
            # static
            self.choices = targs.get('choices', '')
            # dynamic
            self.autofill_deepness = targs.get('autofill_deepness', 3)
            self.autofill_url = targs.get('autofill_url', '')
            self.autofill = targs.get('autofill_related', [])
        return super(StaticSelectMulti, self).__init__(*args, **kwargs)

    def set_language(self, language):
        self.__language = language

    def render(self, name, value, attrs=None, choices=()):
        # Initialization
        # required=self.attrs.get('ng-required','false')
        vmodel = self.field_name
        # vid=attrs.get('id','id_{0}'.format(name))
        # vstyle=attrs.get('style','')
        vform = self.form_name
        if self.static:
            placeholder = escapejs(_('Press * and select an option'))
        else:
            placeholder = _('Press * or start typing')

            if not self.autofill_url:
                raise IOError("autofill_url not defined")

            vurl = reverse(self.autofill_url, kwargs={'search': 'a'})[:-1]
            # Get access to the get_label() method and request for the label of the bound input
            if value:
                func = resolve(vurl + '*').func
                clss = get_class(func)
                if self.__language:
                    clss.language = self.__language
                label = clss().get_label(value)

        if value is None:
            value = []

        valuejs = []

        if self.static:
            init = ''
            if not value:
                value = []
            if hasattr(self, "field"):
                for (key, label) in self.field.choices:
                    if value == key:
                        valuejs.append(u'{{"id":"{0}","label":"{1}"}},'.format(key, escapejs(smart_text(label))))
                    init += u'{{"id":"{0}","label":"{1}"}},'.format(key, escapejs(smart_text(label)))
            elif type(self.choices) == list:

                for (key, label) in self.choices:
                    if value == key:
                        valuejs.append(u'{{"id":"{0}","label":"{1}"}},'.format(key, escapejs(smart_text(label))))
                    init += u'{{"id":"{0}","label":"{1}"}},'.format(key, escapejs(smart_text(label)))
            else:
                # FORCE RELOAD DATAS
                elements = self.choices.queryset
                elements._result_cache = None
                for choice in elements:
                    init += u'{{"id":"{0}","label":"{1}"}},'.format(choice.pk, escapejs(smart_text(choice)))
                    if value and (type(value) is list) and (choice.pk in value):
                        valuejs.append(u'{{"id":"{0}","label":"{1}"}},'.format(int(choice.pk), escapejs(smart_text(choice))))
        else:
            init = "getForeignKeys(http,'{0}',amc_items,{{".format(vurl)
            comma = False
            for field in self.autofill:
                if ':' in field:
                    field_filter = field.split(':')[-1]
                    field = field.split(':')[0]
                else:
                    field_filter = field
                if comma:
                    init += ","
                else:
                    comma = True
                init += "'{0}':{1}.{2}".format(field, vform, field_filter)
            init += "}},'{0}',{0},$select.search,{1})\"".format(vmodel, self.autofill_deepness)

        html = u'<md-chips ng-model="amc_select.{0}" md-autocomplete-snap id="{0}" name="{0}" '.format(vmodel)
        html += u'    md-transform-chip="amc_transformChip($chip)"'
        # html += u'    initial = "loadVegetables()"'
        html += u"    ng-init = 'amc_items.{0} = [".format(vmodel)
        #        {"name":"Broccoli","type":"Brassica","_lowername":"broccoli","_lowertype":"brassica"}
        html += init
        if valuejs:
            # html += u" ng-init = 'amc_select.{0} = [".format(vmodel)
            html += u"]; amc_select.{0} = [".format(vmodel)
            html += ''.join(valuejs)
            html += u"]'"
        html += u"]'"
        html += u'    md-require-match="amc_autocompleteDemoRequireMatch">'
        html += u'    <md-autocomplete'
        html += u'            md-selected-item="amc_selectedItem.{}"'.format(vmodel)
        html += u'            md-search-text="amc_searchText.{}"'.format(vmodel)
        html += u'            md-items="item in amc_querySearch(amc_searchText.{0}, \'{0}\', \'{0}\')"'.format(vmodel)
        html += u'            md-item-text="item.id"'
        html += u'            placeholder="{}">'.format(placeholder)
        html += u'        <span md-highlight-text="amc_searchText.{}">'.format(vmodel)
        html += u'            {{item.label}}</span>'
        html += u'    </md-autocomplete>'
        html += u'    <md-chip-template>'
        html += u'        <span>'
        html += u'        {{$chip.label}}'
        html += u'        </span>'
        html += u'    </md-chip-template>'
        html += u'</md-chips>'
        return html


class StaticSelect(forms.widgets.Select):
    '''
    Howto:
    1) The form class must inherit from GenModelForm
    '''
    def __init__(self, *args, **kwargs):
        targs = None
        if args:
            targs = args[0]
        if kwargs:
            targs = kwargs
        if targs:
            self.is_required = targs.get('is_required', False)
            self.form_name = targs.get('form_name', '')
            self.field_name = targs.get('field_name', '')
            self.choices = targs.get('choices', '')
        return super(StaticSelect, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, choices=()):
        # Initialization
        required = self.attrs.get('ng-required', 'false')
        controller = self.attrs.get('ng-controller', None)
        disabled = self.attrs.get('ng-disabled', None)
        change = self.attrs.get('ng-change', 'undefined').replace('"', "'").replace("'", "\\'")
        if change != "undefined":
            change = "'{}'".format(change)
        vmodel = self.field_name
        vid = attrs.get('id', 'id_{0}'.format(name))
        vstyle = attrs.get('style', '')
        ngreadonly = attrs.get('ng-readonly', '')
        vform = smart_text(self.form_name)
        placeholder = escapejs(_('Select an option'))
        is_multiple = hasattr(self, "multiple") and self.multiple

        # Render
        html = u"<input name='{0}' ng-required='{1}' ng-model='{0}' type='hidden'".format(vmodel, required)
        if controller:
            html += " ng-controller='{}' ".format(controller)
        if disabled:
            html += " ng-disabled='{}' ".format(disabled)
        valuejs = None
        if is_multiple:
            if value:
                if type(value) == list:
                    valuejs = json.dumps([int(x) for x in value])
                else:
                    valuejs = json.dumps([int(value), ])

                html += " ng-init=\"{0}={1}\"".format(vmodel, valuejs)
            else:
                html += " ng-init=\"{0}=[]\"".format(vmodel)
        elif value:
            html += " ng-init=\"{0}='{1}'\"".format(vmodel, value)

        html += ">"

        html += '<ui-select '
        if disabled:
            html += " ng-disabled='{}' ".format(disabled)
        else:
            html += ' ng-disabled="{}"'.format(ngreadonly)
        if controller:
            html += " ng-controller='{}' ".format(controller)
        if is_multiple:
            html += ' multiple '
        # Build options
        html += u" ng-init=\"options.{0}=[".format(vmodel)
        ini = None
        list_value = []
        index = 0

        if is_multiple and not value:
            value = []
        if hasattr(self, "field"):
            for (key, label) in self.field.choices:
                if value == key:
                    ini = index
                html += u"{{'id':'{0}','label':'{1}'}},".format(key, escapejs(smart_text(label)))
                index += 1
        elif type(self.choices) == list:
            for (key, label) in self.choices:
                if value == key:
                    ini = index
                html += u"{{'id':'{0}','label':'{1}'}},".format(key, escapejs(smart_text(label)))
                index += 1
        else:
            if not is_multiple:
                html += u"{{'id':null, 'label':'{0}'}}, ".format(placeholder)

            for choice in self.choices.queryset:
                if value == choice.pk:  # only not multiple
                    ini = index
                html += u"{{'id':'{0}','label':'{1}'}},".format(choice.pk, escapejs(smart_text(choice)))
                if value and (type(value) is list) and (str(choice.pk) in value):
                    list_value.append({'id': int(choice.pk), 'label': str(choice)})
                index += 1

            # raise Exception(html)
        html += "];"
        # Select if we have already one
        if ini is not None:
            if is_multiple:
                html += u"$select.selected=options.{0};".format(vmodel, ini)
            else:
                html += u"$select.placeholder='options.{0}[{1}];'".format(vmodel, ini)
            # ok html += u"$select.selected=options.{0}".format(vmodel,ini)

        if is_multiple and value is not None:
            html += u"{0}.{1}.$setViewValue({2});".format(vform, vmodel, json.dumps(list_value).replace('"', "'"))
            # html += u"$parent.{1}.$setViewValue({2});".format(vform,vmodel,json.dumps(list_value).replace('"', "'"))

        html += ' "'
        # Build the rest of the tag
        html += u" id=\"{0}\"".format(vid)
        html += u" ng-model=\"$parent.{0}\"".format(vmodel)
        if is_multiple:
            html += u" on-select=\"selectedOptionSelect({0}.{1},'{2}',{3}, $parent)\"".format(vform, vmodel, valuejs, change)
        else:
            if value is None:
                value = "null"
            else:
                value = "'{}'".format(value)
            html += u" on-select=\"selectedOptionSelect({0}.{1},{2},{3}, $parent)\"".format(vform, vmodel, value, change)
        html += u' theme="bootstrap"'
        html += u' ng-disabled="disabled"'
        html += u' reset-search-input="false"'
        html += u'>'
        html += u' <ui-select-match placeholder="{}">'.format(placeholder)
        if is_multiple:
            html += u'    {{$item.label}}'
        else:
            html += u'    {{$select.selected.label}}'
        html += u' </ui-select-match>'
        html += u' <ui-select-choices'
        html += u"     style=\"{0}\"".format(vstyle)
        html += u"     repeat=\"value.id as value in options.{0}".format(vmodel)
        html += u'            | filter: {label: $select.search}">'
        html += u'     <div ng-bind-html="value.label| highlightSelect: $select.search"></div>'
        html += u' </ui-select-choices>'
        html += u'</ui-select>'
        # Return result
        return html


class DynamicSelect(forms.widgets.Select):
    '''
    Howto:

    1) The form class must inherit from GenModelForm

    2) In the form class add inside the Meta class a new attribute named autofill:
        autofill = {'ng-model':['select', length, baseurl, field1, field2, field3, ...]}

        Meaning:
        - ng-model: model name in AngularJS
        - 'select': it is constant string 'select'
        - length: how many characters the user has to add in the input so a search will happen
        - baseurl: reverse url name to reach to the class that we will ask for the options to render
        - field1, field2, .... : other ng-model fields that are linked to this one and use to optimize the option's search

    3) You must define an url with a reverse name that we will use to configure baseurl

    4) The target (view) for this url (reverse name) must implement GenForeignKey.

    5) If you use the templatetag 'addattr' to add a 'ng-change' remember you have to operate with $externalScope which referers to the Scope outside the widget, this is usefull when the widget is inside some ng-repeat or another artificial structure generated by the programmer. To operate with form fields you don't need to operate with $externalScope, just point to the form_name.field_name as usually.
    '''
    __language = None

    def __init__(self, *args, **kwargs):
        targs = None
        if args:
            targs = args[0]
        if kwargs:
            targs = kwargs
        if targs:
            self.is_required = targs.get('is_required', False)
            self.form_name = targs.get('form_name', '')
            self.field_name = targs.get('field_name', '')
            self.autofill_deepness = targs.get('autofill_deepness', 3)
            self.autofill_url = targs.get('autofill_url', '')
            self.autofill = targs.get('autofill_related', [])
        return super(DynamicSelect, self).__init__(*args, **kwargs)

    def set_language(self, language):
        self.__language = language

    def __get_foreign(self, vurl, vform, vmodel, search):
        html = "getForeignKeys(http,'{0}',options,{{".format(vurl)
        comma = False
        for field in self.autofill:
            if ':' in field:
                field_filter = field.split(':')[-1]
                field = field.split(':')[0]
            else:
                field_filter = field
            if comma:
                html += ","
            else:
                comma = True
            html += "'{0}':{1}.{2}".format(field, vform, field_filter)
        html += "}}, '{0}', {1}, {2}, {3});\"".format(vmodel, vmodel, search, self.autofill_deepness)
        return html

    def render(self, name, value, attrs=None, choices=()):
        if not self.autofill_url:
            raise IOError("autofill_url not defined")
        # Initialization
        controller = self.attrs.get('ng-controller', None)
        disabled = self.attrs.get('ng-disabled', None)
        required = self.attrs.get('ng-required', 'false')
        change = self.attrs.get('ng-change', 'undefined').replace('"', "'").replace("'", "\\'")
        if change != "undefined":
            change = "'{}'".format(change)
        vmodel = self.field_name
        vid = attrs.get('id', 'id_{0}'.format(name))
        vstyle = attrs.get('style', '')
        vform = smart_text(self.form_name)
        vurl = reverse(self.autofill_url, kwargs={'search': 'a'})[:-1]
        # Get access to the get_label() method and request for the label of the bound input
        if value:
            func = resolve(vurl + '*').func
            clss = get_class(func)
            if self.__language:
                clss.language = self.__language
            label = clss().get_label(value)

        placeholder = _('Press * or start typing')

        # Detect parent node
        vmodelsp = vmodel.split(".")
        if len(vmodelsp) > 1:
            vparent = ".".join(vmodelsp[:-1])
        else:
            vparent = "$parent"

        # Render
        html = u"<input name='{0}' ng-required='{1}' ng-model='{0}' id='{0}' type='hidden'".format(vmodel, required)
        if controller:
            html += " ng-controller='{}' ".format(controller)
        if disabled:
            html += " ng-disabled='{}' ".format(disabled)
        # Set initial value
        if value:
            html += u" ng-init=\"{0}='{1}'\"".format(vmodel, value, vform)
        html += ">"
        html += '<ui-select'
        if controller:
            html += " ng-controller='{}' ".format(controller)
        if disabled:
            html += " ng-disabled='{}' ".format(disabled)
        if hasattr(self, "multiple") and self.multiple:
            html += ' multiple '
        if value:
            html += u" ng-init=\"options.{0}=[{{'id':null,'label':'{3}'}},{{'id':'{1}','label':'{2}'}}]; $select.selected=options.{0}[1];".format(vmodel, value, escapejs(label), placeholder)
            html += u" option_default={}; options.{}=option_default['rows']".format(self.__get_foreign(vurl, vform, vmodel, "'*'"), vmodel)
        else:
            # init options for form modal
            html += u" ng-init=\"options.{0}=[{{'id':null,'label':'{1}'}}]; \"".format(vmodel, placeholder)
        html += u" ng-click=\"option_default={}; options.{}=option_default['rows']".format(self.__get_foreign(vurl, vform, vmodel, "'*'"), vmodel)

        html += u" id=\"{0}\"".format(vid)
        html += u" ng-model=\"{0}.{1}\"".format(vparent, vmodel)
        html += u" on-select=\"selectedOptionSelect({0}.{1},'{2}',{3}, $parent)\"".format(vform, vmodel, value, change)
        html += ' theme="bootstrap"'
        html += ' ng-disabled="disabled"'
        html += ' reset-search-input="false"'
        html += '>'
        html += u' <ui-select-match placeholder="{}">'.format(placeholder)
        html += ' <div ng-bind-html="$select.selected.label"></div>'
        html += ' </ui-select-match>'
        html += ' <ui-select-choices'
        html += u"     style=\"{0}\"".format(vstyle)
        html += u"     repeat=\"value.id as value in options.{0}\"".format(vmodel)
        html += u"     refresh=\"{}\"".format(self.__get_foreign(vurl, vform, vmodel, "$select.search"))
        html += '     refresh-delay="0">'
        html += '     <div ng-bind-html="value.label| highlightSelect: $select.search"></div>'
        html += ' </ui-select-choices>'
        html += '</ui-select>'
        return html


class MultiStaticSelect(StaticSelectMulti):
    static = True


class MultiDynamicSelect(StaticSelectMulti):
    static = False


class MultiStaticSelect_old(StaticSelect):
    multiple = True


class MultiDynamicSelect_old(DynamicSelect):
    multiple = True


class DynamicInput(forms.widgets.Input):
    '''
    Howto:

    1) The form class must inherit from GenModelForm

    2) In the form class add inside the Meta class a new attribute named autofill:
        autofill = {'ng-model':['input', length, baseurl, field1, field2, field3, ...]}

        Meaning:
        - ng-model: model name in AngularJS
        - 'input': it is constant string 'input'
        - length: how many characters the user has to add in the input so a search will happen
        - baseurl: reverse url name to reach to the class that we will ask for the options to render
        - field1, field2, .... : other ng-model fields that are linked to this one and use to optimize the option's search

    3) You must define an url with a reverse name that we will use to configure baseurl

    4) The target (view) for this url (reverse name) must implement GenForeignKey.
    '''

    def render(self, name, value, attrs=None, choices=()):

        # Requet the field to render normally
        html = super(DynamicInput, self).render(name, value, attrs=attrs)

        # Initialization
        vform = self.form_name
        vurl = reverse(self.autofill_url, kwargs={'search': 'a'})[:-1]

        # Render
        ticket = "uib-typeahead=\"item.label for item in getAutoComplete(http,'{0}', {{".format(vurl)
        comma = False
        for field in self.autofill:
            if comma:
                html += ","
            else:
                comma = True
            ticket += "'{0}':{1}.{0}".format(field, vform)
        ticket += "}},$viewValue,{0})\"".format(self.autofill_deepness)
        ticket += ' typeahead-on-select="autoFillFields($item, $model, $label, $event)"'
        ticket += ' typeahead-wait-ms="800"'
        ticket += ' ng-change="resetAutoComplete()"'

        # Save ticket inside html
        html = html.replace('type="None"', 'type="text"')
        html = html.replace('/>', "{0}/>".format(ticket))

        # Return updated html
        return html


class FileAngularInput(forms.widgets.FileInput):
    def render(self, name, value, attrs=None):
        if not attrs:
            attrs = {}
        if value is not None:
            attrs.update({
                "value": value
            })
        if self.is_required:
            attrs.update({
                "required": "required"
            })
        attrs.update({
            "valid-file": "valid-file",
            "base-sixty-four-input": "base-sixty-four-input"
        })

        # Hacer link hacia el archivo
        button = super(FileAngularInput, self).render(name, value, attrs=attrs)
        if value and type(value) != dict:
            if imghdr.what("{}/{}".format(settings.MEDIA_ROOT, value)) is not None:
                image = u'<img src="{0}{1}" style="max-height:75px; max-width:150px;" />'.format(settings.MEDIA_URL, value)
                link = u'<a href="{0}{1}" target="_blank">{2}</a>'.format(settings.MEDIA_URL, value, image)

                html = '<div class="row">'
                html += '   <div class="col-md-6">{}</div>'.format(button)
                html += '   <div class="col-md-6">{}</div>'.format(link)
                html += '</div>'
            else:
                link = u'<br /><a href="{0}{1}" target="_blank">{2}</a><br />'.format(settings.MEDIA_URL, value, value)
                html = link + button
        else:
            html = button

        return html

    def value_from_datadict(self, data, files, name):
        # Get field
        field = data[name]

        if isinstance(field, dict):
            # Prepare filename
            temp_hexname = "{0}{1}".format(field['filename'].encode('ascii', 'ignore'), random.random())
            hexname = hashlib.sha1(temp_hexname.encode()).hexdigest()

            ext = field['filename'].split(".")[-1]
            if not ext:
                ext = "dat"

            # Prepare temporal file in memory
            f = io.BytesIO()
            f.name = "{0}.{1}".format(hexname, ext)
            f.original_name = field['filename']
            f.write(base64.b64decode(field[u'base64']))
            f.seek(0)

            # Add the files to FILES
            files[name] = File(f)

            # Let Django do its work
            return super(FileAngularInput, self).value_from_datadict(data, files, name)
        else:
            return {}


# class Date2TimeInput(forms.widgets.TextInput):
class Date2TimeInput(forms.widgets.DateTimeInput):

    def render(self, name, value, attrs=None):
        if not attrs:
            attrs = {}

        if 'label' in attrs.keys():
            label = attrs.pop('label')
        else:
            label = True

        if self.is_required:
            attrs.update({
                "required": "required"
            })
        attrs.update({
            "class": "form-control ng-valid ng-pristine"
        })

        value_date = ''  # str(datetime.today().date())
        value_time = ''  # '0000'

        try:
            list_type = [str, unicode, ]
        except NameError:
            list_type = [str, ]

        if value and value != "":
            if type(value) == datetime:
                value_date = value.date()
                value_time = '{0:02d}{1:02d}'.format(value.time().hour, value.time().minute)
            elif type(value) in list_type:
                # 2015-06-03 00:00 or 2015-06-03 00:00:00
                try:
                    ###########################################################################
                    # WARNING: I suspect this is a bad patch because it doesn't detect the    #
                    #          format of the data depending on the Django's localization      #
                    #          system. It should checked how is being used in                 #
                    #          method value_from_datadict() from this same class              #
                    ###########################################################################
                    if len(value) == 16:
                        aux = datetime.strptime(value, "%Y-%m-%d %H:%M")
                    elif len(value) == 19:
                        aux = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
                    else:
                        aux = None
                except ValueError:
                    aux = None

                if aux:
                    value_date = aux.date()
                    value_time = '{0:02d}{1:02d}'.format(aux.time().hour, aux.time().minute)

        startview = 2
        minview = 2
        maxview = 4
        icon = 'calendar'

        ##############################################################################
        # WARNING: Language code should be detected on runtime from the user request #
        ##############################################################################
        langcode = settings.LANGUAGE_CODE
        format_date = formats.get_format('DATETIME_INPUT_FORMATS', lang=langcode)[0].replace("%", "").replace('d', 'dd').replace('m', 'mm').replace('Y', 'yyyy').replace('H', 'hh').replace('M', 'ii').split(" ")[0]
        language = settings.LANGUAGE_CODE
        # language = {{LANGUAGE_CODE|default:"en"}}

        if label:
            html = '<label style="margin-right:10px" class="pull-right" for="id_date">'+_('Hour')+'</label>'
        else:
            html = ''
        html += '<div class="row">'
        html += '<div class="col-sm-8">'
        html += ' <div class="dropdown">'
        html += '    <div id="date_{0}" class="input-group date">'.format(name)

        tmp = []
        for x in attrs:
            y = attrs[x]
            tmp.append(x+'="'+y+'"')
        attributes = ' '.join(tmp)

        html += '        <input type="text" name="{0}" id="id_{0}" value="{1}" {2} />'.format(name, value_date, attributes)
        html += '        <span class="input-group-addon"><i class="glyphicon glyphicon-{0}"></i></span>'.format(icon)
        html += '    </div>'
        html += ' </div>'
        html += '</div>'
        html += '<div class="col-sm-4">'

        ngmodel = attrs['ng-model'].split('\']')
        if len(ngmodel) > 1:
            ngmodel[0] = "{}_time".format(ngmodel[0])
            ngmodel = "']".join(ngmodel)
        else:
            ngmodel = attrs['ng-model']+'_time'
        # attrs['ng-model'] = attrs['ng-model']+'_time'
        attrs['ng-model'] = ngmodel
        tmp = []
        for x in attrs:
            y = attrs[x]
            tmp.append(x+'="'+y+'"')
        attributes = ' '.join(tmp)
        html += '        <input type="text" name="{0}_time" id="id_{0}_time" value="{1}" maxlength="4" {2} />'.format(name, value_time, attributes)
        html += '</div>'
        html += '</div>'
        html += '<script type="text/javascript"> '
        html += '$("#date_{0}").datetimepicker({1}'.format(name, '{')
        html += ' format: "{0}",'.format(format_date)
        html += ' autoclose: true,'
        html += ' language:"{0}",'.format(language)
        html += ' todayBtn: true,'
        html += ' weekStart:1,'
        html += ' todayHighlight:true,'
        html += ' pickerPosition:"bottom-left",'
        html += ' keyboardNavigation:false,'
        html += ' startView:{0},'.format(startview)
        html += ' minView:{0},'.format(minview)
        html += ' maxView:{0},'.format(maxview)
        html += ' minuteStep: 15,'
        html += '});'
        html += '</script>'
        return html

    def value_from_datadict(self, data, files, name):
        # Get field
        if name in data:
            date = data[name]
            ttime = '00:00'
            if name+"_time" in data:
                ttime = data[name+"_time"].strip()

                if ttime.isnumeric():
                    if len(ttime) == 0:
                        ttime = '00:00'
                    elif len(ttime) <= 2:
                        ttime = '00:{0}'.format(ttime)
                    elif len(ttime) == 3:
                        ttime = '0{0}:{1}'.format(ttime[:1], ttime[1:])
                    else:
                        ttime = '{0}:{1}'.format(ttime[:2], ttime[2:])

            try:
                new_date = datetime.strptime("{0} {1}".format(date, ttime), formats.get_format('DATETIME_INPUT_FORMATS', lang=get_language())[0])
            except ValueError:
                new_date = None
            # new_date = "{0} {1}".format(date, ttime)
        else:
            new_date = None
            # new_date = ''

        data[name] = new_date
        return data[name]


class WysiwygAngularRender(forms.widgets.HiddenInput):
    def render_wysiwyg(self, ngmodel, extraif="", force_editors=False, attrs=None):
        # Compute hashkey
        hashkey = attrs.get('id', str(random.randint(0, 1000)))
        # Editors
        editors = {}
        editors['quill'] = _("NG Quill")
        editors['textangular'] = _("Text Angular")
        editors['raw'] = _("Source code")
        editors['preview'] = _("Preview")
        editors_list = []
        for keyarg in editors.keys():
            editors_list.append(u"\"{0}\":{{\"key\":\"{0}\",\"name\":\"{1}\"}}".format(keyarg, smart_text(editors[keyarg])))
        editors_json = u"{" + ",".join(editors_list) + "}"

        # Detect if this field is required
        if self.is_required:
            required = ' required'
        else:
            required = ''

        attributes = []
        attributes_ngquill = []
        if attrs:
            for key in attrs.keys():
                if key not in ['id', 'class', 'ng-model', 'ng-init']:
                    attributes.append('{}="{}"'.format(key, attrs[key]))
                    if key in ['ng-blur', 'ng-change']:
                        attributes_ngquill.append(u'on-content-changed="{}"'.format(attrs[key]))
                    attributes_ngquill.append(u'{}="{}"'.format(key, attrs[key]))
        attributes = u" ".join(attributes)
        attributes_ngquill = u" ".join(attributes_ngquill)
        # Get normal field
        html = u""
        html += u"<div ng-init='editors_{1}={0}'></div>".format(editors_json, hashkey)
        # Render wysiwyg editor's selector
        if force_editors:
            ngshow = u""
        else:
            ngshow = u" ng-show='block.type==\"string\"'>"
        html += u"<select ng-model='editor_{1}'{0}>".format(ngshow, hashkey)
        html += u"<option ng-repeat=\"subeditor in editors_{0}\" value=\"{{{{subeditor.key}}}}\">{{{{subeditor.name}}}}</option>".format(hashkey)
        html += u"</select>"

        # Render wysiwyg editors
        html += u"<div ng-if='{1}editor_{2}==\"preview\"' class=\"form-control\" ng-bind-html=\"{0}\"></div>".format(ngmodel, extraif, hashkey)
        html += u"<textarea ng-model=\"{0}\" ng-if='{1}editor_{4}==\"raw\"' class=\"form-control\" rows=\"10\"{2} {3}></textarea>".format(ngmodel, extraif, required, attributes, hashkey)
        html += u"<ng-quill-editor ng-model=\"{0}\" ng-if='{1}editor_{4}==\"quill\"'{2} {3}></ng-quill-editor>".format(ngmodel, extraif, required, attributes_ngquill, hashkey)
        html += u"<text-angular ng-model=\"{0}\" ng-if='{1}editor_{4}==\"textangular\"'{2} {3}></text-angular>".format(ngmodel, extraif, required, attributes, hashkey)
        html += u"<textarea ng-model=\"{0}\" ng-if='{1}editor' style='background-color:#fdd'{2} {3}></textarea>".format(ngmodel, extraif, required, attributes)

        return html


class WysiwygAngularInput(WysiwygAngularRender):
    def render(self, name, value, attrs=None):
        # Compute hashkey
        hashkey = attrs.get('id', str(random.randint(0, 1000)))
        # Get model name
        vmodel = attrs.get('ng-model')  # .replace("'",'"')
        init = attrs.get('ng-init', '')

        if value is None:
            value = ''
        # Render
        html = "<div ng-init='editor_{0}=\"quill\"'>".format(hashkey)
        html += u"<textarea name=\"{0}\" ng-model=\"{1}\" ng-show='false' ng-init=\"{3}\">{2}</textarea>".format(name, vmodel, value, init)
        html += self.render_wysiwyg(ngmodel=vmodel, force_editors=True, attrs=attrs)
        html += "</div>"

        # Return result
        return html


class MultiBlockWysiwygInput(WysiwygAngularRender):
    def render(self, name, value, attrs=None):
        # Compute hashkey
        hashkey = attrs.get('id', str(random.randint(0, 1000)))

        # Get model name
        vmodel = attrs.get('ng-model').replace("'", '"')
        # Get normal field
        html = "<div ng-init='{0}={{}}; {0}[\"__JSON_DATA__\"]={1}'></div>".format(vmodel, value)
        html += u"<input type='hidden' name='{0}' ng-model='{1}'>".format(name, vmodel)

        # Render blocks with ANGULAR
        html += "<div ng-repeat='(key, block) in {0}[\"__JSON_DATA__\"]' ng-init='editor_{1}=\"quill\"'>".format(vmodel, hashkey)
        html += '<label>{{key}}</label>'

        html += '<div ng-show="block.deleted"><p class="text-danger">{}</p></div>'.format(_("Field deleted in the template"))
        html += self.render_wysiwyg(ngmodel='block.value', extraif="block.type==\"string\" && ", attrs=attrs)
        html += "</div>"

        # Return result
        return html


class VisualHTMLInput(forms.widgets.HiddenInput):
    '''
    Example
    service_day_html = forms.CharField(
        label=_("Quota"),  # Optional
        required=False,
        widget=VisualHTMLInput(),
        initial={'selfname': 'service_day_html', 'data': "<div id='<#id#>' ng-init='<#form:service_day_color#>=\"#ff00ff\"' color-contrast='{{<#form:service_day_color#>}}'>{{<#form:service_day_color#>}}</div>"}
    )
    '''

    visual = True

    def render(self, name, value, attrs=None):
        # Compute hashkey
        hashkey = attrs.get('id', str(random.randint(0, 1000)))
        vmodel = attrs.get('ng-model').replace("'", '"')
        selfname = self.attrs.get('selfname', None)
        value = self.attrs.get('data', None)

        # Process all tags
        if value is not None:
            for keydirty in value.split("<#")[1:]:
                key = keydirty.split("#>")[0]
                keysp = key.split(":")
                actor = keysp[0]
                args = keysp[1:]
                if actor == 'form':
                    if selfname:
                        newname = vmodel.replace(selfname, args[0])
                        value = value.replace('<#{}#>'.format(key), newname)
                    else:
                        raise IOError('selfname must be included in the attrs from the widget with the name of the field in the form')
                elif actor == 'id':
                    value = value.replace('<#id#>', hashkey)
                elif actor == 'ngmodel':
                    value = value.replace('<#ngmodel#>', vmodel)
        else:
            raise IOError('data must be included in the attrs from the widget with the name of the field in the form')

        # Return result
        return value


class BootstrapWysiwygInput(forms.widgets.HiddenInput):
    def render(self, name, value, attrs=None):
        # Get model name
        # vmodel = attrs.get('ng-model').replace("'",'"')

        html = _("We are working on this plugin, it will be ready for next version")

        # Return result
        return html


class GenReCaptchaInput(ReCaptcha):

    # Legacy is added to make the Widget to work as it was designed
    # this field should be True when is being analized to answer an API request
    # and json_details is set to True on the query from the remote client
    # The rest of the time legacy should be kept False
    legacy = False

    def __init__(self, *args, **kwargs):
        # Decide name of the field in the POST
        if 'fieldname' in kwargs:
            fieldname = kwargs.pop('fieldname')
        else:
            fieldname = 'g-recaptcha-response'
        self.recaptcha_response_name = fieldname
        self.recaptcha_challenge_name = fieldname

        # Keep going as usually
        return super(GenReCaptchaInput, self).__init__(*args, **kwargs)

#    def __init__(self, public_key=None, use_ssl=None, attrs={}, *args, **kwargs):
#        self.public_key = public_key or settings.RECAPTCHA_PUBLIC_KEY
#        self.use_ssl = use_ssl if use_ssl is not None else getattr(settings, 'RECAPTCHA_USE_SSL', False)
#        self.js_attrs = attrs
#        super(ReCaptcha, self).__init__(*args, **kwargs)

#    def render(self, name, value, attrs=None):
#        return mark_safe(u'%s' % client.displayhtml(self.public_key, self.js_attrs, use_ssl=self.use_ssl))

    def render(self, name, value, attrs=None):
        if self.legacy:
            html = super(GenReCaptchaInput, self).render(name, value, attrs)
        else:
            html = '<input ng-model="{0}" name="{0}" id="id_{0}" type="hidden" ng-required="true">'.format(name)
            html+= '<div vc-recaptcha key="\'{}\'" ng-model="{}"></div>'.format(self.public_key, name)
        return html



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Library to handle lockers over files
'''

__version__ = "2015061200"

import os
import fcntl
import unittest
import tempfile

__all__ = [ "pylock" , "AlreadyLocked" ]

class pylock:
    '''
    Function to control locking flags over a file
    '''

    def __init__(self, lockfile, locktype):
        '''
        Parameters:
        - `lockfile`: name of the file to check/apply the locking.
        - `locktype`: possible values are:
                wait: on a call to lock() function, the system will wait to get the locker
                lock: on a call to lock() function, if locked the system will raise an AlreadyLocked exception
        '''

        # Save config
        self.__lockfile = lockfile
        self.__locktype = locktype
        self.__fd = None

        # Check file exists and create it if it does not
        if not os.path.exists(lockfile):
            file = open(lockfile, 'w')
            file.close()

        # Check locktype
        if locktype not in ['wait','lock']:
            raise TypeError("Locking type unkown")

    def __del__(self):
        '''
        when dying make sure the lock become free
        '''
        # If file was open, close it and delete it!
        if self.__fd:
            self.__fd.close()

    def lock(self):
        '''
        Try to get locked the file
        - the function will wait until the file is unlocked if 'wait' was defined as locktype
        - the funciton will raise AlreadyLocked exception if 'lock' was defined as locktype
        '''

        # Open file
        self.__fd = open(self.__lockfile, "w")

        # Get it locked
        if self.__locktype == "wait":
            # Try to get it locked until ready
            fcntl.flock(self.__fd.fileno(), fcntl.LOCK_EX)
        elif self.__locktype == "lock":
            # Try to get the locker if can not raise an exception
            try:
                fcntl.flock(self.__fd.fileno(), fcntl.LOCK_EX|fcntl.LOCK_NB)
            except IOError:
                raise AlreadyLocked("File is already locked")

    def free(self):
        '''
        Set the locked file free
        '''

        # Close file
        self.__fd.close()

        # Delete it
        try:
            os.unlink(self.__lockfile)
        except:
            pass



# Exceptions classes
class AlreadyLocked(Exception):

    def __init__(self,string):
        self.string = string

    def __str__(self):
        return self.string

# Testing
class test_pylock(unittest.TestCase):
    '''
    Testing class for pylock
    '''

    def setUp(self):
        pass

    def testpylock(self):
        # Get temporal file
        f = tempfile.NamedTemporaryFile(delete=False)
        f.close()

        locker1 = pylock(f.name, 'lock')
        locker1.lock()

        locker2 = pylock(f.name, 'lock')
        self.assertRaises(AlreadyLocked, locker2.lock)
        locker1.free()
        locker2.lock()
        self.assertRaises(AlreadyLocked, locker1.lock)
        locker2.free()

        # Remove the temporal file
        os.unlink(f.name)


# Base call
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.http import HttpResponseRedirect
from django.db import transaction

# Warning: Q objects are used inside an eval() that you can find somewhere down
from django.db.models import Q

class MultiForm(object):
    '''
    model = Flight                                  # Main model
    form_class = FlightForm                         # Main form
    groups = DepartureForm.__groups__(prefix='DepartureForm_')
        + FlightForm.__groups__(prefix="FlightForm_")
        + ArrivalForm.__groups__(prefix='ArrivalForm_') # Define groups, when declared the system will use one only form without tabs
    forms = [                                       # Define several forms together build with elements:
        (DepartureForm, 'flight', None),            # (ModalForm, 'field name used for linking', filters for the queryset ),
        (None, None, None),                         # (None, None, None), -> MainForm
        (ArrivalForm, 'flight', None)               # This definition is used on CREATE
        ]
    forms = [                                       # This definition is used on UPDATE
        (DepartureForm,'flight','kind="D",alternative=False'),
        (None, None, None),
        (ArrivalForm, 'flight','kind="A",alternative=False')
        ]
    '''
    def get_form_kwargs(self):
        kwargs = super(MultiForm, self).get_form_kwargs()
        if 'field_prefix' in self.__dict__:
            kwargs['scope_prefix'] = self.field_prefix
        return kwargs

    def get(self, request, *args, **kwargs):
        """
        Handles GET requests and instantiates blank versions of the form and its inline formsets.
        """

        # Prepare base
        if 'pk' in kwargs:
            self.object = self.get_object()
        else:
            self.object = None
        form_class = self.get_form_class()

        # Get prefix
        if 'field_prefix' in form_class.Meta.__dict__:
            # Get name from the form
            field_prefix = form_class.Meta.field_prefix
        else:
            # Get name from the class
            field_prefix = str(form_class).split("'")[1].split(".")[-1]
        self.field_prefix = field_prefix

        # Build form
        form = self.get_form(form_class)

        # Find groups
        if 'groups' in dir(self):
            # Save groups
            groups = self.groups
            # Redefine groups inside the form
            form.__groups__ = lambda: groups
            # Initialize list of fields
            fields = []
        else:
            groups = None

        # Add special prefix support to properly support form independency
        form.add_prefix = lambda fields_name, field_prefix=field_prefix: "%s_%s" % (field_prefix, fields_name)
        if 'autofill' not in dir(form.Meta):
            form.Meta.autofill = {}

        # For every extra form
        forms = []
        position_form_default = 0
        for (formelement, linkerfield, modelfilter) in self.forms:
            if formelement is None:
                formobj = form
                position_form_default = len(forms)
            else:
                # Locate linked element
                if self.object:
                    related_name = formelement._meta.model._meta.get_field(linkerfield).related_query_name()
                    queryset = getattr(self.object, related_name)
                    if modelfilter:
                        queryset = queryset.filter(eval("Q(%s)" % (modelfilter)))
                    get_method = getattr(queryset, 'get', None)
                    if get_method:
                        instance = queryset.get()
                    else:
                        instance = queryset
                else:
                    instance = None

                if 'autofill' in dir(formelement.Meta):
                    formname = str(formelement).split('.')[-1].split("'")[0]
                    for key in formelement.Meta.autofill:
                        form.Meta.autofill['{}_{}'.format(formname, key)] = formelement.Meta.autofill[key]

                # Get prefix
                if 'field_prefix' in formelement.Meta.__dict__:
                    # Get name from the form
                    field_prefix = formelement.Meta.field_prefix
                else:
                    # Get name from the class
                    field_prefix = str(formelement).split("'")[1].split(".")[-1]
                self.field_prefix = field_prefix

                # Prepare form
                formobj = formelement(instance=instance)
                formobj.form_name = form.form_name

                # Excluded fields
                if 'exclude' not in formobj.Meta.__dict__:
                    formobj.Meta.exclude = [linkerfield]
                elif linkerfield not in formobj.Meta.exclude:
                    formobj.Meta.exclude.append(linkerfield)
                if linkerfield in formobj.fields:
                    del(formobj.fields[linkerfield])

                # Add special prefix support to properly support form independency
                formobj.add_prefix = lambda fields_name, field_prefix=field_prefix: "%s_%s" % (field_prefix, fields_name)
                formobj.scope_prefix = field_prefix

            # Save fields to the list
            if groups:
                for field in formobj:
                    fields.append(field)
            else:
                # Add the form to the list of forms
                forms.append(formobj)

        if position_form_default == 0:
            open_tabs = 1
        else:
            open_tabs = 0
        # Remember list of fields
        if groups:
            form.list_fields = fields

        # Add context and return new context
        return self.render_to_response(self.get_context_data(form=form, forms=forms, open_tabs=open_tabs, position_form_default=position_form_default))

    def post(self, request, *args, **kwargs):
        """
        andles POST requests, instantiating a form instance and its inline formsets with the passed POST variables and then checking them for validity.
        """

        # Prepare base
        if 'pk' in kwargs:
            self.object = self.get_object()
        else:
            self.object = None
        form_class = self.get_form_class()

        # Get prefix
        if 'field_prefix' in form_class.Meta.__dict__:
            # Get name from the form
            field_prefix = form_class.Meta.field_prefix
            # Initialize list of fields
        else:
            # Get name from the class
            field_prefix = str(form_class).split("'")[1].split(".")[-1]
        self.field_prefix = field_prefix

        # Build the form
        form = self.get_form(form_class)

        # Find groups
        if 'groups' in dir(self):
            # Save groups
            groups = self.groups
            # Redefine groups inside the form
            form.__groups__ = lambda: groups
            # Initialize list of fields
            fields = []
        else:
            groups = None

        # Add special prefix support to properly support form independency
        form.add_prefix = lambda fields_name, field_prefix=field_prefix: "%s_%s" % (field_prefix, fields_name)

        # Check validation
        valid = form.is_valid()
        if (not valid) and ('non_field_errors' in dir(self)):
            errors = [element[5] for element in list(self.non_field_errors())[:-1]]
        elif form.errors.as_data():
            errors = []
            for element in form.errors.as_data():
                for err in form.errors.as_data()[element][0]:
                    errors.append(err)
        else:
            errors = []

        # For every extra form
        temp_forms = []
        position_form_default = 0
        for (formelement, linkerfield, modelfilter) in self.forms:
            if formelement is None:
                formobj = form
                position_form_default = len(temp_forms)
            else:
                # Locate linked element
                if self.object:
                    related_name = formelement._meta.model._meta.get_field(linkerfield).related_query_name()
                    queryset = getattr(self.object, related_name)
                    if modelfilter:
                        queryset = queryset.filter(eval("Q(%s)" % (modelfilter)))
                    get_method = getattr(queryset, 'get', None)
                    if get_method:
                        instance = queryset.get()
                    else:
                        instance = queryset
                else:
                    instance = None

                # Get prefix
                if 'field_prefix' in formelement.Meta.__dict__:
                    # Get name from the form
                    field_prefix = formelement.Meta.field_prefix
                else:
                    # Get name from the class
                    field_prefix = str(formelement).split("'")[1].split(".")[-1]
                self.field_prefix = field_prefix

                # Prepare form
                formobj = formelement(instance=instance, data=self.request.POST)
                formobj.form_name = form.form_name

                # Excluded fields
                if 'exclude' not in formobj.Meta.__dict__:
                    formobj.Meta.exclude = [linkerfield]
                elif linkerfield not in formobj.Meta.exclude:
                    formobj.Meta.exclude.append(linkerfield)
                if linkerfield in formobj.fields:
                    del(formobj.fields[linkerfield])

                # Link it to the main model
                formobj.add_prefix = lambda fields_name, field_prefix=field_prefix: "%s_%s" % (field_prefix, fields_name)

                # Validate
                valid *= formobj.is_valid()

                # append error
                if not formobj.is_valid() and ('non_field_errors' in dir(formobj)):
                    errors += [element[5] for element in list(formobj.non_field_errors())[:-1]]

            # Save fields to the list
            if groups:
                for field in formobj:
                    # raise Exception (field.__dict__)
                    if 'unblock_t2ime' in field.html_name:
                        raise Exception(field.field.__dict__)
                    fields.append(field)

            # Add a new form
            temp_forms.append((formobj, linkerfield))

        # execute validation specified
        validate_forms = None
        if valid and ("validate" in dir(self)):
            validate_forms = [tform[0] for tform in temp_forms]
            errors = self.validate(*validate_forms)
            # valid = len(errors) == 0
            valid = False
            if errors is None or len(errors) == 0:
                valid = True

        # Remember list of fields
        if groups:
            form.list_fields = fields
            forms = []
        else:
            if validate_forms:
                forms = validate_forms
            else:
                forms = [tform[0] for tform in temp_forms]

        if position_form_default == 0:
            open_tabs = 1
        else:
            open_tabs = 0

        # Check validation result
        if valid:
            # Everything is OK, call valid
            return self.form_valid(form, temp_forms)
        else:
            # Something went wrong, attach error and call invalid
            form.list_errors = errors
            return self.form_invalid(form, forms, open_tabs, position_form_default)

    @transaction.atomic
    def form_valid(self, form, forms):
        """
        Called if all forms are valid. Creates a Recipe instance along with associated Ingredients and Instructions and then redirects to a success page.
        """
        if self.object:
            form.save()
            for (formobj, linkerfield) in forms:
                if form != formobj:
                    formobj.save()
        else:
            self.object = form.save()
            for (formobj, linkerfield) in forms:
                if form != formobj:
                    setattr(formobj.instance, linkerfield, self.object)
                    formobj.save()
        return HttpResponseRedirect(self.get_success_url())

    def form_invalid(self, form, forms, open_tabs, position_form_default):
        """
        Called if a form is invalid. Re-renders the context data with the data-filled forms and errors.
        """
        # return self.render_to_response( self.get_context_data( form = form, forms = forms ) )
        return self.render_to_response(self.get_context_data(form=form, forms=forms, open_tabs=open_tabs, position_form_default=position_form_default))



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from bson import json_util
import json
from django.db import models
from django.apps import apps
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.db.models import Q
from django.core.exceptions import ValidationError
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import force_text, smart_text
from django.conf import settings
from django.db.models.signals import post_delete, pre_delete
from django.dispatch.dispatcher import receiver
from django.core.exceptions import PermissionDenied

from codenerix.middleware import get_current_user
from codenerix.helpers import daterange_filter

# Separator to log
SEPARATOR = u'\u8594'
SEPARATOR_HTML = ' &rarr; '


class CodenerixMetaType(dict):
    """
    Define type for CodenerixMeta of the instance NOT the class
    Example:
    m = CodenerixMetaType({'first_name': 'Eduardo'}, last_name='Pool', age=24, sports=['Soccer'])
    """
    def __init__(self, *args, **kwargs):
        super(CodenerixMetaType, self).__init__(*args, **kwargs)
        for arg in args:
            if isinstance(arg, dict):
                for k in arg:
                    self[k] = arg[k]

        if kwargs:
            for k in kwargs:
                self[k] = kwargs[k]

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(CodenerixMetaType, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(CodenerixMetaType, self).__delitem__(key)
        del self.__dict__[key]

    def __getnewargs__(self):
        return tuple()

    def __getstate__(self):
        return self.__dict__


class CodenerixModelBase(models.Model):
    class Meta:
        abstract = True

    # return method relation objects
    def __getmro__(self):
        return self.__class__.__mro__

    # recolecta informacion de todas las clases que intervienen en la instancia
    # collects information from all classes that intervene in the instance
    def __init__(self, *args, **kwards):
        self.CodenerixMeta = CodenerixMetaType()

        mro = self.__getmro__()
        for cl in reversed(mro):
            if 'CodenerixMeta' in cl.__dict__.keys():
                for key in cl.CodenerixMeta.__dict__.keys():
                    if '__' != key[0:2]:
                        value = getattr(cl.CodenerixMeta, key)
                        if value:
                            if key not in self.CodenerixMeta:
                                self.CodenerixMeta[key] = value
                            else:
                                if type(value) == dict:
                                    self.CodenerixMeta[key].update(value)
                                elif type(value) == list:
                                    if type(self.CodenerixMeta[key]) != list:
                                        self.CodenerixMeta[key] = list(self.CodenerixMeta[key])
                                    self.CodenerixMeta[key] += value
                                elif type(self.CodenerixMeta[key]) == list:
                                    self.CodenerixMeta[key] += list(value)

        return super(CodenerixModelBase, self).__init__(*args, **kwards)


class CodenerixModel(CodenerixModelBase):
    '''
    Special methods are
        __fields__: it is a list of fields
            Usage:      fields.append(('key','Name',size:int_in_pixels,'alignment:left|right|center'))
            Example 1:  fields.append(('title',_('Title')))
            Example 2:  fields.append(('title',_('Title'),100,'center'))
            Example 3:  fields.append(('title',_('Title'),None,'center'))   # We don't want to define the size but we want to define the alignment
            Example 4:  fields.append((None,_('Title')))                    # We don't want any ordering in the field Title
            Example 5:  fields.append(('user__username',_('Username')))     # You can define here relationships as well

        __limitQ__:
        __searchQ__:
        __searchF__:
    '''
    created = models.DateTimeField(_("Created"), editable=False, auto_now_add=True)
    updated = models.DateTimeField(_("Updated"), editable=False, auto_now=True)

    class Meta:
        abstract = True
        default_permissions = ('add', 'change', 'delete', 'view', 'list')

    class CodenerixMeta(object):
        abstract = None

    def __init__(self, *args, **kwards):
        self.CodenerixMeta = CodenerixMetaType()
        return super(CodenerixModel, self).__init__(*args, **kwards)

    def __strlog_add__(self):
        return ''

    def __strlog_update__(self, newobj):
        return ''

    def __strlog_delete__(self):
        return ''

    def __limitQ__(self, info):
        return {}

    def __searchQ__(self, info, text):
        return {}

    def __searchF__(self, info):
        return {}

    def lock_update(self, request=None):
        return None

    def internal_lock_delete(self):
        answer = self.lock_delete()
        if answer is None:
            for related in self._meta.get_fields():
                if 'on_delete' in related.__dict__ and related.on_delete == models.PROTECT:
                    field = getattr(self, related.related_name, None)
                    if field and field.exists():
                        answer = _('Cannot delete item, relationship with %(model_name)s') % {'model_name': related.related_model._meta.verbose_name}
                        break
        return answer

    def lock_delete(self, request=None):
        return None

    # check lock update
    def clean(self):
        if self.lock_update() is not None:
            raise ValidationError(self.lock_update())
        else:
            return super(CodenerixModel, self).clean()


class GenInterface(CodenerixModelBase):
    """
    Check force_methods options in CodenerixMeta class and it makes sure that the specified methods exists
    """
    class Meta:
        abstract = True

    class CodenerixMeta(object):
        """
        force_methods = {'alias': ('method_name', 'Description'), }
        """
        pass

    def __init__(self, *args, **kwards):
        self.CodenerixMeta = CodenerixMetaType()
        super(GenInterface, self).__init__(*args, **kwards)

        # revisamos que esten implementados los metodos indicados
        # we checked that the indicated methods are implemented
        force_methods = getattr(self.CodenerixMeta, "force_methods", None)
        if force_methods:
            for alias in force_methods.keys():
                method = force_methods[alias]
                if not hasattr(self, method[0]) or not callable(getattr(self, method[0])):
                    raise IOError("Method {}() not found in class {}: {}".format(method[0], self._meta.object_name, method[1]))
        return super(GenInterface, self).__init__(*args, **kwards)


@receiver(pre_delete)
def codenerixmodel_delete_pre(sender, instance, **kwargs):
    if not hasattr(instance, "name_models_list") and hasattr(instance, 'internal_lock_delete') and callable(instance.internal_lock_delete):
        lock_delete = instance.internal_lock_delete()
        if lock_delete is not None:
            raise PermissionDenied(lock_delete)


# We don't use log system when PQPRO_CASSANDRA == TRUE
if not (hasattr(settings, "PQPRO_CASSANDRA") and settings.PQPRO_CASSANDRA):
    from django.contrib.admin.models import ADDITION, CHANGE, DELETION

    TYPE_ACTION = (
        (ADDITION, _("Add")),
        (CHANGE, _("Change")),
        (DELETION, _("Delete")),
    )

    class Log(models.Model):
        '''
        Control the possible log
        '''
        action_time = models.DateTimeField('Date', auto_now=True)
        user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
        content_type = models.ForeignKey(ContentType, blank=True, null=True)
        object_id = models.TextField('Object id', blank=True, null=True)
        object_repr = models.CharField('Object repr', max_length=200)
        action_flag = models.PositiveSmallIntegerField(_("Action"), choices=TYPE_ACTION)
        change_json = models.TextField('Json', blank=True, null=False)
        change_txt = models.TextField('Txt', blank=True, null=False)
        snapshot_txt = models.TextField('Snapshot Txt', blank=True, null=False)

        class Meta:
            permissions = (
                ("list_log", "Can list log"),
                ("detail_log", "Can view log"),
            )

        def show(self, view='html'):
            text = []
            if self.change_txt:
                cambios = json.loads(self.change_txt)
            else:
                cambios = ''
            for c in cambios:
                if type(cambios[c]) is list:
                    text.append(u"{}: {}".format(cambios[c][0], cambios[c][1]))
                else:
                    text.append(u"{}: {}".format(_(c), cambios[c]))

            if view == 'html':
                result = mark_safe(u'<ul><li>{}</li></ul>'.format(u'</li><li>'.join(text)).replace(SEPARATOR, SEPARATOR_HTML))
            else:
                result = '\n'.join(text)

            return result

        def __unicode__(self):
            return self.show(view='txt')

        def __str__(self):
            return self.__unicode__()

        def action(self):
            # Find the action
            if self.action_flag == ADDITION:
                answer = _("Add")
            elif self.action_flag == CHANGE:
                answer = _("Change")
            elif self.action_flag == DELETION:
                answer = _("Delete")
            else:
                answer = "?"
            # Return answer
            return answer

        def __fields__(self, info):
            fields = []
            fields.append(('action_time', _('Date')))
            fields.append(('user__username', _('User')))
            fields.append(('get_action_flag_display', _('Action')))
            # fields.append(('content_type__name', _('APP Name')))
            fields.append(('content_type', _('APP Name')))
            fields.append(('content_type__app_label', _('APP Label')))
            fields.append(('content_type__model', _('APP Model')))
            fields.append(('object_id', _('ID')))
            # fields.append(('object_repr', _('Representation')))
            fields.append(('show', _('Txt')))
            return fields

        def __searchQ__(self, info, text):
            tf = {}
            tf['user'] = Q(user__username__icontains=text)
            # Flag
            if (text.lower() == 'add') or (text.lower() == 'addition') or (text.lower() == _('add')) or (text.lower() == _('addition')):
                tf['action_flag'] = Q(action_flag=ADDITION)
            elif (text.lower() == 'change') or (text.lower() == 'changed') or (text.lower() == _('change')) or (text.lower() == _('changed')):
                tf['action_flag'] = Q(action_flag=CHANGE)
            elif (text.lower() == 'edit') or (text.lower() == 'edition') or (text.lower() == _('edit')) or (text.lower() == _('edition')):
                tf['action_flag'] = Q(action_flag=CHANGE)
            elif (text.lower() == 'delete') or (text.lower() == 'deleted') or (text.lower() == _('delete')) or (text.lower() == _('deleted')):
                tf['action_flag'] = Q(action_flag=DELETION)
            # tf['content_type'] = Q(content_type__name__icontains=text)
            tf['object_id'] = Q(object_id__icontains=text)
            tf['object_repr'] = Q(object_repr__icontains=text)
            tf['action_time'] = 'datetime'
            return tf

        def __searchF__(self, info):
            tf = {}
            tf['action_time'] = (_('Date'), lambda x: Q(**daterange_filter(x, 'action_time')), 'daterange')
            tf['get_action_flag_display'] = (_('Action'), lambda x: Q(action_flag=x), list(TYPE_ACTION))
            tf['object_id'] = (_('ID'), lambda x: Q(object_id=x), 'input')
            tf['user__username'] = (_('User'), lambda x: Q(user__username__icontains=x), 'input')
            tf['content_type__app_label'] = (_('APP Label'), lambda x: Q(content_type__app_label__icontains=x), 'input')
            # tf['users']=(_('User'),lambda x: Q(user__username=x),[('M','M*'),('S','S*')])
            return tf

    class GenLog(object):

        class CodenerixMeta(CodenerixModel.CodenerixMeta):
            log_full = False

        def post_save(self, log):
            # custom post save from application
            pass

        def save(self, **kwargs):
            user = get_current_user()
            if user:
                user_id = user.pk
            else:
                user_id = None

            model = apps.get_model(self._meta.app_label, self.__class__.__name__)
            isnew = True
            if self.pk is not None:
                list_obj = model.objects.filter(pk=self.pk)
                isnew = list_obj.count() == 0
                # raise IOError,self.__dict__
            # only attributes changes
            attrs = {}
            attrs_txt = {}
            # attributes from database
            attrs_bd = {}
            if isnew:
                action = ADDITION
                pk = None
            else:
                action = CHANGE
                pk = self.pk
                # Instance object
                # obj = model.objects.get(pk=self.pk)
                obj = list_obj.get()
                for key in obj._meta.get_fields():
                    key = key.name
                    # exclude manytomany
                    if obj._meta.model._meta.local_many_to_many and key in [x.name for x in obj._meta.model._meta.local_many_to_many]:
                        value = None
                    elif obj._meta.get_fields(include_hidden=True) and key in [x.name for x in obj._meta.get_fields(include_hidden=True) if x.many_to_many and x.auto_created]:
                        value = None
                    else:
                        value = getattr(obj, key, None)
                    attrs_bd[key] = value

            # comparison attributes
            # for key in self._meta.get_fields():
            aux = None
            list_fields = [x.name for x in self._meta.get_fields()]
            for ffield in self._meta.get_fields():
                key = ffield.name
                # exclude manytomany
                if self._meta.model._meta.local_many_to_many and key in [x.name for x in self._meta.model._meta.local_many_to_many]:
                    field = None
                # elif self._meta.get_all_related_many_to_many_objects() and key in [x.name for x in self._meta.get_all_related_many_to_many_objects()]:
                elif self._meta.get_fields(include_hidden=True) and key in [x.name for x in self._meta.get_fields(include_hidden=True) if x.many_to_many and x.auto_created]:
                    field = None
                else:
                    field = getattr(self, key, None)

                if key in list_fields:
                    # if (not attrs_bd.has_key(key)) or (field != attrs_bd[key]):
                    if (key not in attrs_bd) or (field != attrs_bd[key]):
                        if field is not None or action == CHANGE:
                            aux = ffield
                            field_txt = field
                            if field_txt is None:
                                field_txt = '---'
                            if isinstance(field, CodenerixModel):
                                field = field.pk

                            try:
                                json.dumps(field, default=json_util.default)
                                if key not in attrs_bd or not self.CodenerixMeta.log_full:
                                    attrs[key] = field
                                else:
                                    if isinstance(attrs_bd[key], CodenerixModel):
                                        attrs[key] = (attrs_bd[key].pk, field, )
                                    else:
                                        attrs[key] = (attrs_bd[key], field, )
                            except Exception:
                                # If related, we don't do anything
                                if getattr(field, 'all', None) is None:
                                    field = str(field)
                                    if key not in attrs_bd or not self.CodenerixMeta.log_full:
                                        attrs[key] = field
                                    else:
                                        attrs[key] = (attrs_bd[key], field, )

                            if hasattr(ffield, "verbose_name"):
                                try:
                                    string_checks = [unicode, str]
                                except NameError:
                                    string_checks = [str]

                                if type(ffield.verbose_name) in string_checks:
                                    ffield_verbose_name = ffield.verbose_name
                                else:
                                    ffield_verbose_name = str(ffield.verbose_name)

                                if key not in attrs_bd or not self.CodenerixMeta.log_full:
                                    attrs_txt[ffield_verbose_name] = force_text(field_txt, errors='replace')
                                else:
                                    if attrs_bd[key] is None:
                                        attrs_bd[key] = '---'
                                    attrs_txt[key] = (
                                        ffield_verbose_name,
                                        u"{}{}{}".format(
                                            force_text(attrs_bd[key], errors='replace'),
                                            SEPARATOR,
                                            force_text(field_txt, errors='replace')
                                        )
                                    )

            log = Log()
            log.user_id = user_id
            log.content_type_id = ContentType.objects.get_for_model(self).pk
            log.object_id = pk
            log.object_repr = force_text(self, errors="replace")
            try:
                log.change_json = json.dumps(attrs, default=json_util.default)
            except UnicodeDecodeError:
                log.change_json = json.dumps({'error': '*JSON_ENCODE_ERROR*'}, default=json_util.default)
            try:
                log.change_txt = json.dumps(attrs_txt, default=json_util.default)
            except UnicodeDecodeError:
                log.change_txt = json.dumps({'error': '*JSON_ENCODE_ERROR*'}, default=json_util.default)
            log.action_flag = action
            if pk is None:
                log.snapshot_txt = self.__strlog_add__()
            else:
                log.snapshot_txt = obj.__strlog_update__(self)

            aux = super(GenLog, self).save(**kwargs)
            if pk is None:
                # if new element, get pk
                log.object_id = self.pk
            log.save()

            # custom post save from application
            self.post_save(log)

            return aux

    class GenLogFull(GenLog):
        class CodenerixMeta(CodenerixModel.CodenerixMeta):
            log_full = True

    @receiver(post_delete)
    def codenerixmodel_delete_post(sender, instance, **kwargs):
        if not hasattr(instance, "name_models_list") and issubclass(sender, GenLog):
            user = get_current_user()
            if user:
                user_id = user.pk
            else:
                user_id = None
            action = DELETION

            attrs = {}
            attrs_txt = {}

            # ._meta.get_fields() return all fields include related name
            # ._meta.fields return all fields of models

            # list_fields = [x.name for x in instance._meta.get_fields()]
            for ffield in instance._meta.get_fields():
                key = ffield.name
                # exclude manytomany
                if instance._meta.model._meta.local_many_to_many and key in [x.name for x in instance._meta.model._meta.local_many_to_many]:
                    field = None
                # elif self._meta.get_all_related_many_to_many_objects() and key in [x.name for x in self._meta.get_all_related_many_to_many_objects()]:
                elif instance._meta.get_fields(include_hidden=True) and key in [x.name for x in instance._meta.get_fields(include_hidden=True) if x.many_to_many and x.auto_created]:
                    field = None
                else:
                    field = getattr(instance, key, None)

                value = getattr(instance, ffield.name)
                if isinstance(value, CodenerixModel):
                    attrs[ffield.name] = value.pk
                else:
                    try:
                        json.dump(value, default=json_util.default)
                        attrs[ffield.name] = value
                    except TypeError:
                        # If related, we don't do anything
                        if getattr(value, 'all', None) is None:
                            # value = str(value)
                            value = smart_text(value)
                            attrs[ffield.name] = value

                if hasattr(ffield, "verbose_name"):
                    try:
                        string_checks = [unicode, str]
                    except NameError:
                        string_checks = [str]

                    if type(ffield.verbose_name) in string_checks:
                        ffield_verbose_name = ffield.verbose_name
                    else:
                        ffield_verbose_name = str(ffield.verbose_name)

                    attrs_txt[ffield_verbose_name] = force_text(field, errors='replace')

            log = Log()
            log.user_id = user_id
            log.content_type_id = ContentType.objects.get_for_model(instance).pk
            log.object_id = instance.pk
            log.object_repr = force_text(instance)
            log.change_json = json.dumps(attrs, default=json_util.default)
            log.change_txt = json.dumps(attrs_txt, default=json_util.default)
            log.snapshot_txt = instance.__strlog_delete__()
            log.action_flag = action
            log.save()

    class RemoteLog(CodenerixModel):
        '''
        RemoteLog system
        '''
        user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
        data = models.TextField('Data', blank=False, null=False)

        def __fields__(self, info):
            fields = []
            fields.append(('pk', _('ID')))
            fields.append(('created', _('Created')))
            fields.append(('user', _('Username')))
            return fields



import os
import sys
from setuptools import setup

import codenerix

#from setuptools.command.install import install
#class CustomInstallCommand(install):
#    """Customized setuptools install command - prints a friendly greeting."""
#    def run(self):
#        print "Hello, developer, how are you? :)"
#        install.run(self)
#        #post-processing code

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

requirements = codenerix.__requirements__['all']
if sys.version_info[0] < 3:
    requirements += codenerix.__requirements__['2']
else:
    requirements += codenerix.__requirements__['3']

setup(
    name='django-codenerix',
    version=codenerix.__version__,
    packages=['codenerix'],
    include_package_data=True,
    zip_safe=False,
    license='Apache License Version 2.0',
    description='Codenerix it is a framework that goes on top of Django so it makes easier development and building of ERPs.',
    long_description=README,
    url='https://github.com/centrologic/django-codenerix',
    author=', '.join(codenerix.__authors__),
    keywords=['django', 'codenerix', 'management','erp','crm'],
    platforms=['OS Independent'],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Intended Audience :: Developers',
        'License :: Other/Proprietary License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    # cmdclass={ 'install': CustomInstallCommand, },
    install_requires = requirements,
)




# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-19 15:26
# Simplified by Juanmi Taboada on 2017-14-02 11:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0007_corporateimage'),
    ]

    operations = []



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from djng.forms.angular_model import NgModelFormMixin
from djng.forms import NgFormValidationMixin, NgForm, NgModelForm

from django.utils.translation import ugettext as _
from django.forms.widgets import Select, CheckboxInput
from django.forms import NullBooleanField

from codenerix.helpers import model_inspect
from codenerix.widgets import StaticSelect, DynamicSelect, DynamicInput, MultiStaticSelect, MultiDynamicSelect


class BaseForm(object):

    __language = None
    attributes = {}

    def set_language(self, language):
        self.__language = language

    def set_attribute(self, key, value):
        self.attributes[key] = value

    def get_name(self):
        # If name atrribute exists in Meta
        if 'name' in self.Meta.__dict__:
            # Try to get name from Meta
            name = self.Meta.name
        else:
            # If not try to find it automatically
            info = model_inspect(self.Meta.model())
            if info['verbose_name']:
                name = info['verbose_name']
            else:
                name = info['modelname']
        return name

    def __errors__(self):
        return []

    def clean_color(self):
        color = self.cleaned_data["color"]
        if len(color) != 0:
            valores_validos = "#0123456789abcdefABCDEF"
            r = True
            for l in color:
                if l not in valores_validos:
                    r = False
                    break
            if not r or color[0] != "#" or not (len(color) == 4 or len(color) == 7):
                self._errors["color"] = [_("Invalid color")]
                return color
            else:
                return color
        else:
            return color

    def get_errors(self):
        # Where to look for fields
        if 'list_errors' in dir(self):
            list_errors = self.list_errors
        else:
            # r = self.non_field_errors()
            # list_errors = [element[5] for element in list(self.non_field_errors())[:-1]]
            list_errors = []
            for element in list(self.non_field_errors())[:-1]:
                if len(element) >= 5:
                    list_errors.append(element[5])
        return list_errors

    def __groups__(self):
        return []

    def get_groups(self, gs=None, processed=[], initial=True):
        '''
        <--------------------------------------- 12 columns ------------------------------------>
                    <--- 6 columns --->                           <--- 6 columns --->
         ------------------------------------------   ------------------------------------------
        | Info                                     | | Personal                                 |
        |==========================================| |==========================================|
        |  -----------------   ------------------  | |                                          |
        | | Passport        | | Name             | | | Phone                          Zipcode   |
        | |=================| | [.....]  [.....] | | | [...........................]  [.......] |
        | | CID     Country | | <- 6 ->  <- 6 -> | | |       <--- 8 columns --->      <-4 col-> |
        | | [.....] [.....] | |                  | | |                                          |
        | | <- 6 -> <- 6 -> |  -----------------   | | Address                                  |
        |  -----------------                       | | [.....................................]  |
         ------------------------------------------  |           <--- 12 columns --->           |
                                                     | [..] number                              |
                                                     |           <--- 12 columns --->           |
                                                     |                                          |
                                                      ------------------------------------------
        group = [
                (_('Info'),(6,'#8a6d3b','#fcf8e3','center'),
                    (_('Identification'),6,
                        ["cid",6],
                        ["country",6],
                    ),
                    (None,6,
                        ["name",None,6],
                        ["surname",None,6,False],
                    ),
                ),
                (_('Personal'),6,
                    ["phone",None,8],
                    ["zipcode",None,4],
                    ["address",None,12],
                    ["number",None,12, True],
                ),
            ]

        Group: it is defined as tuple with 3 or more elements:
            Grammar: (<Name>, <Attributes>, <Element1>, <Element2>, ..., <ElementN>)
            If <Name> is None: no name will be given to the group and no panel decoration will be shown
            If <Size in columns> is None: default of 6 will be used

            <Attributes>:
                it can be an integer that represent the size in columns
                it can be a tuple with several attributes where each element represents:
                    (<Size in columns>,'#<Font color>','#<Background color>','<Alignment>')

            <Element>:
                it can be a Group
                it can be a Field

            Examples:
            ('Info', 6, ["name",6], ["surname",6]) -> Info panel using 6 columns with 2 boxes 6 columns for each with name and surname inputs
            ('Info', (6,None,'#fcf8e3','center'), ["name",6], ["surname",6]) -> Info panel using 6 columns with a yellow brackground in centered title, 2 boxes, 6 columns for each with name and surname inputs
            ('Info', 12, ('Name', 6, ["name",12]), ('Surname',6, ["surname",12])) -> Info panel using 12 columns with 2 panels inside
              of 6 columns each named "Name" and "Surname" and inside each of them an input "name" and "surname" where it belongs.

        Field: must be a list with at least 1 element in it:
            Grammar: [<Name of field>, <Size in columns>, <Label>]

            <Name of field>:
                This must be filled always
                It is the input's name inside the form
                Must exists as a form element or as a grouped form element

            <Size in columns>:
                Size of the input in columns
                If it is not defined or if it is defined as None: default of 6 will be used

            <Label>:
                It it is defined as False: the label for this field will not be shown
                If it is not defined or if it is defined as None: default of True will be used (default input's label will be shown)
                If it is a string: this string will be shown as a label

            Examples:
            ['age']                             Input 'age' will be shown with 6 columns and its default label
            ['age',8]                           Input 'age' will be shown with 8 columns and its default label
            ['age', None, False]                Input 'age' will be shown with 6 columns and NO LABEL
            ['age',8,False]                     Input 'age' will be shown with 8 columns and NO LABEL
            ['age',8,_("Age in days")]          Input 'age' will be shown with 8 columns and translated label text "Age in days" to user's language
            ['age',8,_("Age in days"), True]    Input 'age' will be shown with 8 columns and translated label text "Age in days" to user's language, and input inline with label
            ['age',6, None, None, None, None, None, ["ng-click=functionjs('param1')", "ng-change=functionjs2()"]]    Input 'age' with extras functions
            ['age',None,None,None,None, 'filter']    Input 'age' with extras filter ONLY DETAILS
            ['age',6, {'color': 'red'}          Input 'age' will be shown with red title
        '''

        # Check if language is set
        if not self.__language:
            raise IOError("ERROR: No language suplied!")

        # Initialize the list
        if initial:
            processed = []
        # Where to look for fields
        if 'list_fields' in dir(self):
            list_fields = self.list_fields
            check_system = "html_name"
        else:
            list_fields = self
            check_system = "name"

        # Default attributes for fields
        attributes = [
            ('columns', 6),
            ('color', None),
            ('bgcolor', None),
            ('textalign', None),
            ('inline', False),  # input in line with label
            ('label', True),
            ('extra', None),
            ('extra_div', None),
            ('foreign_info', {}),
        ]
        labels = [x[0] for x in attributes]

        # Get groups if none was given
        if gs is None:
            gs = self.__groups__()

        # Prepare the answer
        groups = []

        # Prepare focus control
        focus_first = None
        focus_must = None

        # html helper for groups and fields
        html_helper = self.html_helper()

        # Start processing
        for g in gs:
            token = {}
            token['name'] = g[0]

            if token['name'] in html_helper:
                if 'pre' in html_helper[token['name']]:
                    token["html_helper_pre"] = html_helper[token['name']]['pre']
                if 'post' in html_helper[token['name']]:
                    token["html_helper_post"] = html_helper[token['name']]['post']

            styles = g[1]
            if type(styles) is tuple:
                if len(styles) >= 1:
                    token['columns'] = g[1][0]
                if len(styles) >= 2:
                    token['color'] = g[1][1]
                if len(styles) >= 3:
                    token['bgcolor'] = g[1][2]
                if len(styles) >= 4:
                    token['textalign'] = g[1][3]
                if len(styles) >= 5:
                    token['inline'] = g[1][4]
                if len(styles) >= 7:
                    token['extra'] = g[1][5]
                if len(styles) >= 8:
                    token['extra_div'] = g[1][6]
            else:
                token['columns'] = g[1]
            fs = g[2:]
            fields = []
            for f in fs:
                # Field
                atr = {}
                # Decide weather this is a Group or not
                if type(f) == tuple:
                    # Recursive
                    fields += self.get_groups([list(f)], processed, False)
                else:
                    try:
                        list_type = [str, unicode, ]
                    except NameError:
                        list_type = [str, ]
                    # Check if it is a list
                    if type(f) == list:
                        # This is a field with attributes, get the name
                        field = f[0]

                        if html_helper and token['name'] in html_helper and 'items' in html_helper[token['name']] and field in html_helper[token['name']]['items']:
                            if 'pre' in html_helper[token['name']]['items'][field]:
                                atr["html_helper_pre"] = html_helper[token['name']]['items'][field]['pre']
                            if 'post' in html_helper[token['name']]['items'][field]:
                                atr["html_helper_post"] = html_helper[token['name']]['items'][field]['post']

                        # Process each attribute (if any)
                        dictionary = False
                        for idx, element in enumerate(f[1:]):
                            if type(element) == dict:
                                dictionary = True
                                for key in element.keys():
                                    if key in labels:
                                        atr[key] = element[key]
                                    else:
                                        raise IOError("Unknown attribute '{0}' as field '{1}' in list of fields".format(key, field))
                            else:
                                if not dictionary:
                                    if element is not None:
                                        atr[attributes[idx][0]] = element
                                else:
                                    raise IOError("We already processed a dicionary element in this list of fields, you can not add anoother type of elements to it, you must keep going with dictionaries")
                    elif type(f) in list_type:
                        field = f
                    else:
                        raise IOError("Uknown element type '{0}' inside group '{1}'".format(type(f), token['name']))

                    # Get the Django Field object
                    found = None
                    for infield in list_fields:
                        if infield.__dict__[check_system] == field:
                            found = infield
                            break

                    if found:
                        # Get attributes (required and original attributes)
                        wrequired = found.field.widget.is_required
                        wattrs = found.field.widget.attrs
                        # Fill base attributes
                        atr['name'] = found.html_name
                        atr['input'] = found
                        atr['focus'] = False
                        # Set focus
                        if focus_must is None:
                            if focus_first is None:
                                focus_first = atr
                            if wrequired:
                                focus_must = atr
                        # Autocomplete
                        if 'autofill' in dir(self.Meta):
                            autofill = self.Meta.autofill.get(found.html_name, None)
                            atr['autofill'] = autofill
                            if autofill:
                                # Check format of the request
                                autokind = autofill[0]
                                if type(autokind) == str:
                                    # Using new format
                                    if autokind == 'select':
                                        # If autofill is True for this field set the DynamicSelect widget
                                        found.field.widget = DynamicSelect(wattrs)
                                    elif autokind == 'multiselect':
                                        # If autofill is True for this field set the DynamicSelect widget
                                        found.field.widget = MultiDynamicSelect(wattrs)
                                    elif autokind == 'input':
                                        # If autofill is True for this field set the DynamicSelect widget
                                        found.field.widget = DynamicInput(wattrs)
                                    else:
                                        raise IOError("Autofill filled using new format but autokind is '{}' and I only know 'input' or 'select'".format(autokind))

                                    # Configure the field
                                    found.field.widget.is_required = wrequired
                                    found.field.widget.form_name = self.form_name
                                    found.field.widget.field_name = infield.html_name
                                    found.field.widget.autofill_deepness = autofill[1]
                                    found.field.widget.autofill_url = autofill[2]
                                    found.field.widget.autofill = autofill[3:]
                                else:
                                    # Get old information [COMPATIBILITY WITH OLD VERSION]
                                    # If autofill is True for this field set the DynamicSelect widget
                                    found.field.widget = DynamicSelect(wattrs)
                                    found.field.widget.is_required = wrequired
                                    found.field.widget.form_name = self.form_name
                                    found.field.widget.field_name = infield.html_name
                                    found.field.widget.autofill_deepness = autofill[0]
                                    found.field.widget.autofill_url = autofill[1]
                                    found.field.widget.autofill = autofill[2:]
                        else:

                            # Set we don't have autofill for this field
                            atr['autofill'] = None

                        # Check if we have to replace the widget with a newer one
                        if isinstance(found.field.widget, Select) and not isinstance(found.field.widget, DynamicSelect):
                            if not isinstance(found.field.widget, MultiStaticSelect):
                                found.field.widget = StaticSelect(wattrs)
                            found.field.widget.choices = found.field.choices
                            found.field.widget.is_required = wrequired
                            found.field.widget.form_name = self.form_name
                            found.field.widget.field_name = infield.html_name

                        # Fill all attributes
                        for (attribute, default) in attributes:
                            if attribute not in atr.keys():
                                atr[attribute] = default
                        # Fill label
                        if atr['label'] is True:
                            atr['label'] = found.label
                        # Set language
                        flang = getattr(found.field, "set_language", None)
                        if flang:
                            flang(self.__language)
                        flang = getattr(found.field.widget, "set_language", None)
                        if flang:
                            flang(self.__language)
                        # Attach the element
                        fields.append(atr)
                        # Remember we have processed it
                        processed.append(found.__dict__[check_system])
                    else:
                        raise IOError("Unknown field '{0}' specified in group '{1}'".format(f, token['name']))

            token['fields'] = fields
            groups.append(token)

        # Add the rest of attributes we didn't use yet
        if initial:
            fields = []
            for infield in list_fields:
                if infield.__dict__[check_system] not in processed:
                    # Get attributes (required and original attributes)
                    wattrs = infield.field.widget.attrs
                    wrequired = infield.field.widget.is_required
                    # Prepare attr
                    atr = {}
                    # Fill base attributes
                    atr['name'] = infield.html_name
                    atr['input'] = infield
                    atr['focus'] = False
                    # Set focus
                    if focus_must is None:
                        if focus_first is None:
                            focus_first = atr
                        if wrequired:
                            focus_must = atr
                    # Autocomplete
                    if 'autofill' in dir(self.Meta):
                        autofill = self.Meta.autofill.get(infield.html_name, None)
                        atr['autofill'] = autofill
                        if autofill:
                            # Check format of the request
                            autokind = autofill[0]
                            if type(autokind) == str:
                                # Get old information

                                # Using new format
                                if autokind == 'select':
                                    # If autofill is True for this field set the DynamicSelect widget
                                    infield.field.widget = DynamicSelect(wattrs)
                                elif autokind == 'multiselect':
                                    # If autofill is True for this field set the DynamicSelect widget
                                    infield.field.widget = MultiDynamicSelect(wattrs)
                                elif autokind == 'input':
                                    # If autofill is True for this field set the DynamicSelect widget
                                    infield.field.widget = DynamicInput(wattrs)
                                else:
                                    raise IOError("Autofill filled using new format but autokind is '{}' and I only know 'input' or 'select'".format(autokind))

                                # Configure the field
                                infield.field.widget.is_required = wrequired
                                infield.field.widget.form_name = self.form_name
                                infield.field.widget.field_name = infield.html_name
                                infield.field.widget.autofill_deepness = autofill[1]
                                infield.field.widget.autofill_url = autofill[2]
                                infield.field.widget.autofill = autofill[3:]
                            else:
                                # Get old information [COMPATIBILITY WITH OLD VERSION]
                                # If autofill is True for this field set the DynamicSelect widget
                                infield.field.widget = DynamicSelect(wattrs)
                                infield.field.widget.is_required = wrequired
                                infield.field.widget.form_name = self.form_name
                                infield.field.widget.field_name = infield.html_name
                                infield.field.widget.autofill_deepness = autofill[0]
                                infield.field.widget.autofill_url = autofill[1]
                                infield.field.widget.autofill = autofill[2:]
                    else:

                        # Set we don't have autofill for this field
                        atr['autofill'] = None

                    # Check if we have to replace the widget with a newer one
                    if isinstance(infield.field.widget, Select) and not isinstance(infield.field.widget, DynamicSelect):
                        if isinstance(infield.field, NullBooleanField):
                            infield.field.widget = CheckboxInput(wattrs)
                        elif not isinstance(infield.field.widget, MultiStaticSelect):
                            infield.field.widget = StaticSelect(wattrs)
                        if hasattr(infield.field.widget, 'choices') and hasattr(infield.field, 'choices'):
                            infield.field.widget.choices = infield.field.choices
                        infield.field.widget.is_required = wrequired
                        infield.field.widget.form_name = self.form_name
                        infield.field.widget.field_name = infield.html_name

                    # Fill all attributes
                    for (attribute, default) in attributes:
                        if attribute not in atr.keys():
                            atr[attribute] = default
                    # Fill label
                    if atr['label'] is True:
                        atr['label'] = infield.label
                    # Set language
                    flang = getattr(infield.field, "set_language", None)
                    if flang:
                        flang(self.__language)
                    flang = getattr(infield.field.widget, "set_language", None)
                    if flang:
                        flang(self.__language)
                    # Attach the attribute
                    fields.append(atr)

            # Save the new elements
            if fields:
                groups.append({'name': None, 'columns': 12, 'fields': fields})

        # Set focus
        if focus_must:
            focus_must['focus'] = True
        elif focus_first is not None:
            focus_first['focus'] = True

        # Return the resulting groups
        return groups

    def html_helper(self):
        """
        g={'Group Name':{
            'pre': 'text pre div',
            'post': 'text post div',
            'items': {
                'input name':{'pre': 'text pre name', 'post': 'text post name'},
                'example':{'pre': '<p>text <b>for</b> help</p>', 'post': '<div>more help</div>'},
                }
            }
        }
        """
        return {}


class GenModelForm(BaseForm, NgModelFormMixin, NgFormValidationMixin, NgModelForm):
    pass


class GenForm(BaseForm, NgFormValidationMixin, NgForm):
    add_djng_error = False

    class Meta:
        name = ""



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-05 08:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix', '0011_delete_corporateimage'),
    ]

    operations = [
        migrations.AlterField(
            model_name='log',
            name='action_flag',
            field=models.PositiveSmallIntegerField(choices=[(1, 'A\xf1adir'), (2, 'Modificar'), (3, 'Eliminar')], verbose_name='Acci\xf3n'),
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template import Library
from django.core.urlresolvers import reverse
from django.utils.encoding import smart_text
from django.utils.safestring import mark_safe
from django.core.exceptions import ValidationError
from django.utils import formats
from django.conf import settings

from djng.forms.angular_base import TupleErrorList

from codenerix.helpers import model_inspect

register = Library()

@register.filter
def widgetize(i):
    # Initialize structure
    attrs = i.__dict__.get("field", {}).__dict__.get("widget", {}).__dict__.get('attrs', {})

    # Select
    # if 'choices' in i.field.widget.__dict__:
    #
    #    # Set classes for select2 inputs to look properly with and without foreignkeys link button
    #    if foreignkey(i,""):
    #        addattr(attrs,"class=select_fk")
    #    else:
    #        addattr(attrs,"class=select_nofk")
    #    # Add a new attribute for ui-select to work
    #    addattr(attrs,"ui-select2")

    # Return result
    return attrs


@register.filter
def istype(i, kind):
    # Get widget
    widget = i.field.widget

    # Get format type
    if ('format_key' in type(widget).__dict__):
        ftype = type(widget).format_key
    else:
        ftype = None

    # Choose kind
    if kind == 'datetime':
        if   ftype == 'DATETIME_INPUT_FORMATS':   answer = 'DATETIME_INPUT_FORMATS'
        elif ftype == 'DATE_INPUT_FORMATS':       answer = 'DATE_INPUT_FORMATS'
        elif ftype == 'TIME_INPUT_FORMATS':       answer = 'TIME_INPUT_FORMAT'
        else:                                     answer = False
    elif kind == 'date2time':
        answer = 'DATE_INPUT_FORMATS'
    elif kind == 'color':
        answer = (ngmodel(i) == 'color')
    else:
        IOError("Unknown type '{0}' in 'istype' filter".format(kind))

    # Return answer
    return answer


@register.filter
def addextra(attrs, attr):
    if attr:
        for at in attr:
            addattr(attrs, at)
    # Return result
    return attrs


@register.filter
def addattr(attrs, attr):
    # Split the new attr into key/value pair
    attrsp = attr.split("=")
    key = attrsp[0]
    if len(attrsp) >= 2:
        value = "=".join(attrsp[1:])
    else:
        value = ""

    if key in attrs:
        # Key already exists in the attrs struct
        if attrs[key]:
            # Key has a value already inside the structure
            if value:
                # We got a new value to add to the struct, append it
                attrs[key] += " {0}".format(value)
        else:
            # Key doesn't have a value inside the structure
            if value:
                # We got a new value to add eo the struct, add it
                attrs[key] += value
    else:
        # Add the new key
        attrs[key] = value

    # Return result
    return attrs


@register.filter
def lockattr(attrs, cannot_update):
    if cannot_update:
        if "ui-select2" in attrs:
            attrs.pop("ui-select2")
        newattrs = addattr(attrs, "readonly='readonly'")
        return addattr(newattrs, "disabled='disabled'")
    else:
        return attrs


@register.filter
def setattrs(field, attrs):
    if attrs:
        return field.as_widget(attrs=attrs)
    else:
        return field


@register.filter
def ngmodel(i):
    return getattr(i.field.widget, 'field_name', i.field.widget.attrs['ng-model'])


@register.filter
def inireadonly(attrs, i):
    field = ngmodel(i)
    return addattr(attrs, 'ng-readonly=readonly_{0}'.format(field))


@register.filter
def date2timewidget(i, langcode):
    return datewidget(i, langcode, 'date2time')


@register.filter
def datewidget(i, langcode, kindtype='datetime', kind=None):
    # Initialization
    final = {}
    form = formats.get_format('DATETIME_INPUT_FORMATS', lang=langcode)[0].replace("%", "").replace('d', 'dd').replace('m', 'mm').replace('Y', 'yyyy').replace('H', 'hh').replace('M', 'ii').replace('S', 'ss')

    if kind is None:
        kind = istype(i, kindtype)

    if kind == 'DATETIME_INPUT_FORMATS':
        final['format'] = form
        final['startview'] = 2
        final['minview'] = 0
        final['maxview'] = 4
        final['icon'] = 'calendar'

    elif (kind == 'DATE_INPUT_FORMATS') or (kind == 'date'):
        final['format'] = form.split(" ")[0]
        final['startview'] = 2
        final['minview'] = 2
        final['maxview'] = 4
        final['icon'] = 'calendar'

    elif kind == 'TIME_INPUT_FORMAT':
        final['format'] = form.split(" ")[1]
        final['startview'] = 1
        final['minview'] = 0
        final['maxview'] = 1
        final['icon'] = 'time'

    else:
        raise IOError("Unknown kind '{0}' in filter 'datewidget'".format(kind))

    # Return result
    return final


@register.filter
def unlist(elements):
    # Remake the tuple
    newtuple = TupleErrorList()
    # Process each errror
    for error in elements:
        # Split errors
        (f1, f2, f3, f4, f5, msg) = error
        if type(msg) == ValidationError:
            newmsg = ""
            for error in msg:
                if newmsg:
                    newmsg += " {0}".format(error)
                else:
                    newmsg = error
            # Save new msg
            msg = newmsg
        # Save error with converted text
        newtuple.append((f1, f2, f3, f4, f5, msg))
    # Return the newtuple
    return newtuple


@register.filter
def foreignkey(element, exceptions):
    '''
    function to determine if each select field needs a create button or not
    '''
    label = element.field.__dict__['label']
    try:
        label = unicode(label)
    except NameError:
        pass
    if (not label) or (label in exceptions):
        return False
    else:
        return "_queryset" in element.field.__dict__


@register.filter
def headstyle(group):
    # Initialize
    style = ""

    # Decide about colors
    if 'color' in group and group['color']:
        style += "color:{0};".format(group['color'])
    if 'bgcolor' in group and group['bgcolor']:
        style += "background-color:{0};".format(group['bgcolor'])
    if 'textalign' in group and group['textalign']:
        style += "text-align:{0};".format(group['textalign'])

    # Check if we have some style
    if style:
        return "style={0}".format(style)
    else:
        return ""


class ColumnCounter:
    def __init__(self):
        self.__columns = 0

    def add(self, columns):
        # Control columns
        if self.__columns == 12:
            self.__columns = 0
            answer = True
        elif self.__columns > 12:
            raise IOError("Columns max number of 12 reached, you requested to use a total of '{}'".format(self.__columns))
        else:
            answer = False
        # Add new columns
        self.__columns += columns
        # Return answer
        return answer


@register.filter
def column_counter(nothing):
    return ColumnCounter()


@register.filter
def add_columns(obj, columns):
    return obj.add(columns)


@register.filter
def linkedinfo(element, info_input={}):
    info = model_inspect(element.field._get_queryset().model())
    info.update(info_input)

    ngmodel = element.html_name  # field.widget.attrs['ng-model']
    return mark_safe("'{0}','{1}','{2}', '{3}s'".format(
        getattr(settings, 'BASE_URL', ''),
        ngmodel,
        info['appname'],
        info['modelname'].lower())
    )


# DEPRECATED: 2017-02-14
@register.filter
def get_depa(queryset, kind):
    return queryset.get(kind=kind, alternative=False)


@register.filter
def getws(form, input_name):
    if 'autofill' in form.Meta.__dict__ and input_name in form.Meta.autofill:
        return "'{}'".format(reverse(form.Meta.autofill[input_name][2], kwargs={'search': '__pk__'}))
    else:
        return 'undefined'


@register.filter
def get_field_list(forms):
    inputs = []
    for form in forms:
        for field in form.fields:
            inputs.append("'{}'".format(field))
    if inputs:
        inputs = "[{}]".format(','.join(inputs))
    return inputs


@register.filter
def invalidator(formname, inp):
    return mark_safe("{{'codenerix_invalid':{0}.{1}.$invalid}}".format(smart_text(formname), ngmodel(inp)))


@register.filter
def join_list(l, string):
    if l:
        return string.join(l)
    else:
        return ''



# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

try:
    from subprocess import getstatusoutput
    pythoncmd="python3"
except:
    from commands import getstatusoutput
    pythoncmd="python2"

from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from codenerix.lib.debugger import Debugger
from codenerix.lib.colors import colors

class Command(BaseCommand, Debugger):

    # Show this when the user types help
    help = "Show colors for Debugger"
    
    def handle(self, *args, **options):
        
        # Autoconfigure Debugger
        self.set_name("CODENERIX")
        self.set_debug()
        
        # Reorder colors
        keys = []
        for key in colors.keys():
            keys.append((colors[key][0],colors[key][1],key))
        keys.sort()
        
        # Show up all colors
        for color in keys:
            # Get the color information
            (simplebit, subcolor) = colors[color[2]]
            # Show it
            print("{0:1d}:{1:02d} - \033[{2:1d};{3:02d}m{4:<14s}\033[1;00m{5:<s}".format(simplebit, subcolor, simplebit, subcolor, color[2], color[2]))
            
            # Get environment
            appname=settings.ROOT_URLCONF.split(".")[0]
            basedir=settings.BASE_DIR
            appdir = os.path.abspath("{}/{}".format(basedir,appname))
            status, output = getstatusoutput("find {}/ -type f -name '*.pyc' -delete".format(appdir))
            if status: raise CommandError(output)




# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-25 05:39
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('codenerix', '0019_auto_20170725_1822'),
    ]

    operations = [
        migrations.CreateModel(
            name='RemoteLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('data', models.TextField(verbose_name='Data')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
    ]
